#----------------------------------------------------------------------
# Name:         multisash
# Purpose:      Multi Sash control
#
# Author:       Gerrit van Dyk
#
# Created:      2002/11/20
# Version:      0.1
# License:      wxWindows license
#----------------------------------------------------------------------
# 12/09/2003 - Jeff Grimmett (grimmtooth@softhome.net)
#
# o 2.5 compatibility update.
#
# 12/20/2003 - Jeff Grimmett (grimmtooth@softhome.net)
#
# o wxMultiSash -> MultiSash
# o wxMultiSplit -> MultiSplit
# o wxMultiViewLeaf -> MultiViewLeaf
#

import wx

import six
MV_HOR = 0
MV_VER = not MV_HOR

SH_SIZE = 5
CR_SIZE = SH_SIZE * 3

#----------------------------------------------------------------------

class MultiSash(wx.Window):
    def __init__(self, *_args,**_kwargs):
        wx.Window.__init__(self, *_args, **_kwargs)
        self._defChild = EmptyChild
        self.child = MultiSplit(self,self,(0,0),self.GetSize())
        self.Bind(wx.EVT_SIZE,self.OnMultiSize)

    def SetDefaultChildClass(self,childCls):
        self._defChild = childCls
        self.child.DefaultChildChanged()

    def OnMultiSize(self,evt):
        self.child.SetSize(self.GetSize())

    def UnSelect(self):
        self.child.UnSelect()

    def Clear(self):
        old = self.child
        self.child = MultiSplit(self,self,(0,0),self.GetSize())
        old.Destroy()
        self.child.OnSize(None)

    def GetSaveData(self):
        saveData = {}
        saveData['_defChild_class'] = self._defChild.__name__
        saveData['_defChild_mod']   = self._defChild.__module__
        saveData['child'] = self.child.GetSaveData()
        return saveData

    def SetSaveData(self,data):
        mod = data['_defChild_mod']
        dChild = mod + '.' + data['_defChild_class']
        six.exec_('import %s' % mod)
        self._defChild = eval(dChild)
        old = self.child
        self.child = MultiSplit(self,self,wx.Point(0,0),self.GetSize())
        self.child.SetSaveData(data['child'])
        old.Destroy()
        self.OnMultiSize(None)
        self.child.OnSize(None)


#----------------------------------------------------------------------


class MultiSplit(wx.Window):
    def __init__(self,multiView,parent,pos,size,view1 = None):
        wx.Window.__init__(self,id = -1,parent = parent,pos = pos,size = size,
                          style = wx.CLIP_CHILDREN)
        self.multiView = multiView
        self.view2 = None
        if view1:
            self.view1 = view1
            self.view1.Reparent(self)
            self.view1.Move(0,0)
        else:
            self.view1 = MultiViewLeaf(self.multiView,self,
                                         (0,0),self.GetSize())
        self.direction = None

        self.Bind(wx.EVT_SIZE,self.OnSize)

    def GetSaveData(self):
        saveData = {}
        if self.view1:
            saveData['view1'] = self.view1.GetSaveData()
            if isinstance(self.view1,MultiSplit):
                saveData['view1IsSplit'] = 1
        if self.view2:
            saveData['view2'] = self.view2.GetSaveData()
            if isinstance(self.view2,MultiSplit):
                saveData['view2IsSplit'] = 1
        saveData['direction'] = self.direction
        v1,v2 = self.GetPosition()
        saveData['x'] = v1
        saveData['y'] = v2
        v1,v2 = self.GetSize()
        saveData['w'] = v1
        saveData['h'] = v2
        return saveData

    def SetSaveData(self,data):
        self.direction = data['direction']
        self.SetSize(int(data['x']), int(data['y']), int(data['w']), int(data['h']))
        v1Data = data.get('view1',None)
        if v1Data:
            isSplit = data.get('view1IsSplit',None)
            old = self.view1
            if isSplit:
                self.view1 = MultiSplit(self.multiView,self,
                                          (0,0),self.GetSize())
            else:
                self.view1 = MultiViewLeaf(self.multiView,self,
                                             (0,0),self.GetSize())
            self.view1.SetSaveData(v1Data)
            if old:
                old.Destroy()
        v2Data = data.get('view2',None)
        if v2Data:
            isSplit = data.get('view2IsSplit',None)
            old = self.view2
            if isSplit:
                self.view2 = MultiSplit(self.multiView,self,
                                          (0,0),self.GetSize())
            else:
                self.view2 = MultiViewLeaf(self.multiView,self,
                                             (0,0),self.GetSize())
            self.view2.SetSaveData(v2Data)
            if old:
                old.Destroy()
        if self.view1:
            self.view1.OnSize(None)
        if self.view2:
            self.view2.OnSize(None)

    def UnSelect(self):
        if self.view1:
            self.view1.UnSelect()
        if self.view2:
            self.view2.UnSelect()

    def DefaultChildChanged(self):
        if not self.view2:
            self.view1.DefaultChildChanged()

    def AddLeaf(self,direction,caller,pos):
        if self.view2:
            if caller == self.view1:
                self.view1 = MultiSplit(self.multiView,self,
                                          caller.GetPosition(),
                                          caller.GetSize(),
                                          caller)
                self.view1.AddLeaf(direction,caller,pos)
            else:
                self.view2 = MultiSplit(self.multiView,self,
                                          caller.GetPosition(),
                                          caller.GetSize(),
                                          caller)
                self.view2.AddLeaf(direction,caller,pos)
        else:
            self.direction = direction
            w,h = self.GetSize()
            if direction == MV_HOR:
                x,y = (pos,0)
                w1,h1 = (w-pos,h)
                w2,h2 = (pos,h)
            else:
                x,y = (0,pos)
                w1,h1 = (w,h-pos)
                w2,h2 = (w,pos)
            self.view2 = MultiViewLeaf(self.multiView, self, (x,y), (w1,h1))
            self.view1.SetSize((w2,h2))
            self.view2.OnSize(None)

    def DestroyLeaf(self,caller):
        if not self.view2:              # We will only have 2 windows if
            return                      # we need to destroy any
        parent = self.GetParent()       # Another splitview
        if parent == self.multiView:    # We'r at the root
            if caller == self.view1:
                old = self.view1
                self.view1 = self.view2
                self.view2 = None
                old.Destroy()
            else:
                self.view2.Destroy()
                self.view2 = None
            self.view1.SetSize(self.GetSize())
            self.view1.Move(self.GetPosition())
        else:
            w,h = self.GetSize()
            x,y = self.GetPosition()
            if caller == self.view1:
                if self == parent.view1:
                    parent.view1 = self.view2
                else:
                    parent.view2 = self.view2
                self.view2.Reparent(parent)
                self.view2.SetSize(x,y,w,h)
            else:
                if self == parent.view1:
                    parent.view1 = self.view1
                else:
                    parent.view2 = self.view1
                self.view1.Reparent(parent)
                self.view1.SetSize(x,y,w,h)
            self.view1 = None
            self.view2 = None
            self.Destroy()

    def CanSize(self,side,view):
        if self.SizeTarget(side,view):
            return True
        return False

    def SizeTarget(self,side,view):
        if self.direction == side and self.view2 and view == self.view1:
            return self
        parent = self.GetParent()
        if parent != self.multiView:
            return parent.SizeTarget(side,self)
        return None

    def SizeLeaf(self,leaf,pos,side):
        if self.direction != side:
            return
        if not (self.view1 and self.view2):
            return
        if pos < 10: return
        w,h = self.GetSize()
        if side == MV_HOR:
            if pos > w - 10: return
        else:
            if pos > h - 10: return
        if side == MV_HOR:
            self.view1.SetSize(0,0,pos,h)
            self.view2.SetSize(pos,0,w-pos,h)
        else:
            self.view1.SetSize(0,0,w,pos)
            self.view2.SetSize(0,pos,w,h-pos)

    def OnSize(self,evt):
        if not self.view2:
            self.view1.SetSize(self.GetSize())
            self.view1.OnSize(None)
            return
        v1w,v1h = self.view1.GetSize()
        v2w,v2h = self.view2.GetSize()
        v1x,v1y = self.view1.GetPosition()
        v2x,v2y = self.view2.GetPosition()
        w,h = self.GetSize()

        if v1x != v2x:
            ratio = float(w) / float((v1w + v2w))
            v1w *= ratio
            v2w = w - v1w
            v2x = v1w
        else:
            v1w = v2w = w

        if v1y != v2y:
            ratio = float(h) / float((v1h + v2h))
            v1h *= ratio
            v2h = h - v1h
            v2y = v1h
        else:
            v1h = v2h = h

        self.view1.SetSize(int(v1x), int(v1y), int(v1w), int(v1h))
        self.view2.SetSize(int(v2x), int(v2y), int(v2w), int(v2h))
        self.view1.OnSize(None)
        self.view2.OnSize(None)


#----------------------------------------------------------------------


class MultiViewLeaf(wx.Window):
    def __init__(self,multiView,parent,pos,size):
        wx.Window.__init__(self,id = -1,parent = parent,pos = pos,size = size,
                          style = wx.CLIP_CHILDREN)
        self.multiView = multiView

        self.sizerHor = MultiSizer(self,MV_HOR)
        self.sizerVer = MultiSizer(self,MV_VER)
        self.creatorHor = MultiCreator(self,MV_HOR)
        self.creatorVer = MultiCreator(self,MV_VER)
        self.detail = MultiClient(self,multiView._defChild)
        self.closer = MultiCloser(self)

        self.Bind(wx.EVT_SIZE,self.OnSize)

        self.SetBackgroundColour(wx.SystemSettings.GetColour(wx.SYS_COLOUR_3DFACE))


    def GetSaveData(self):
        saveData = {}
        saveData['detailClass_class'] = self.detail.child.__class__.__name__
        saveData['detailClass_mod'] = self.detail.child.__module__
        if hasattr(self.detail.child,'GetSaveData'):
            attr = getattr(self.detail.child,'GetSaveData')
            if callable(attr):
                dData = attr()
                if dData:
                    saveData['detail'] = dData
        v1,v2 = self.GetPosition()
        saveData['x'] = v1
        saveData['y'] = v2
        v1,v2 = self.GetSize()
        saveData['w'] = v1
        saveData['h'] = v2
        return saveData

    def SetSaveData(self,data):
        mod = data['detailClass_mod']
        dChild = mod + '.' + data['detailClass_class']
        six.exec_('import %s' % mod)
        detClass = eval(dChild)
        self.SetSize(data['x'],data['y'],data['w'],data['h'])
        old = self.detail
        self.detail = MultiClient(self,detClass)
        dData = data.get('detail',None)
        if dData:
            if hasattr(self.detail.child,'SetSaveData'):
                attr = getattr(self.detail.child,'SetSaveData')
                if callable(attr):
                    attr(dData)
        old.Destroy()
        self.detail.OnSize(None)

    def UnSelect(self):
        self.detail.UnSelect()

    def DefaultChildChanged(self):
        self.detail.SetNewChildCls(self.multiView._defChild)

    def AddLeaf(self,direction,pos):
        if pos < 10: return
        w,h = self.GetSize()
        if direction == MV_VER:
            if pos > h - 10: return
        else:
            if pos > w - 10: return
        self.GetParent().AddLeaf(direction,self,pos)

    def DestroyLeaf(self):
        self.GetParent().DestroyLeaf(self)

    def SizeTarget(self,side):
        return self.GetParent().SizeTarget(side,self)

    def CanSize(self,side):
        return self.GetParent().CanSize(side,self)

    def OnSize(self,evt):
        def doresize():
            try:
                self.sizerHor.OnSize(evt)
                self.sizerVer.OnSize(evt)
                self.creatorHor.OnSize(evt)
                self.creatorVer.OnSize(evt)
                self.detail.OnSize(evt)
                self.closer.OnSize(evt)
            except:
                pass
        wx.CallAfter(doresize)

#----------------------------------------------------------------------


class MultiClient(wx.Window):
    def __init__(self,parent,childCls):
        w,h = self.CalcSize(parent)
        wx.Window.__init__(self,id = -1,parent = parent,
                          pos = (0,0),
                          size = (w,h),
                          style = wx.CLIP_CHILDREN | wx.SUNKEN_BORDER)
        self.child = childCls(self)
        self.child.Move(2,2)
        self.normalColour = self.GetBackgroundColour()
        self.selected = False

        self.Bind(wx.EVT_SET_FOCUS,self.OnSetFocus)
        self.Bind(wx.EVT_CHILD_FOCUS,self.OnChildFocus)

    def UnSelect(self):
        if self.selected:
            self.selected = False
            self.SetBackgroundColour(self.normalColour)
            self.Refresh()

    def Select(self):
        self.GetParent().multiView.UnSelect()
        self.selected = True
        self.SetBackgroundColour(wx.Colour(255,255,0)) # Yellow
        self.Refresh()

    def CalcSize(self,parent):
        w,h = parent.GetSize()
        w -= SH_SIZE
        h -= SH_SIZE
        return (w,h)

    def OnSize(self,evt):
        w,h = self.CalcSize(self.GetParent())
        self.SetSize(0,0,w,h)
        w,h = self.GetClientSize()
        self.child.SetSize((w-4,h-4))

    def SetNewChildCls(self,childCls):
        if self.child:
            self.child.Destroy()
            self.child = None
        self.child = childCls(self)
        self.child.Move(2,2)

    def OnSetFocus(self,evt):
        self.Select()

    def OnChildFocus(self,evt):
        self.OnSetFocus(evt)
##        from Funcs import FindFocusedChild
##        child = FindFocusedChild(self)
##        child.Bind(wx.EVT_KILL_FOCUS,self.OnChildKillFocus)


#----------------------------------------------------------------------


class MultiSizer(wx.Window):
    def __init__(self,parent,side):
        self.side = side
        x,y,w,h = self.CalcSizePos(parent)
        wx.Window.__init__(self,id = -1,parent = parent,
                          pos = (x,y),
                          size = (w,h),
                          style = wx.CLIP_CHILDREN)

        self.px = None                  # Previous X
        self.py = None                  # Previous Y
        self.isDrag = False             # In Dragging
        self.dragTarget = None          # View being sized

        self.Bind(wx.EVT_LEAVE_WINDOW,self.OnLeave)
        self.Bind(wx.EVT_ENTER_WINDOW,self.OnEnter)
        self.Bind(wx.EVT_MOTION,self.OnMouseMove)
        self.Bind(wx.EVT_LEFT_DOWN,self.OnPress)
        self.Bind(wx.EVT_LEFT_UP,self.OnRelease)

        self.SetBackgroundColour(wx.SystemSettings.GetColour(wx.SYS_COLOUR_3DFACE))


    def CalcSizePos(self,parent):
        pw,ph = parent.GetSize()
        if self.side == MV_HOR:
            x = CR_SIZE + 2
            y = ph - SH_SIZE
            w = pw - CR_SIZE - SH_SIZE - 2
            h = SH_SIZE
        else:
            x = pw - SH_SIZE
            y = CR_SIZE + 2 + SH_SIZE
            w = SH_SIZE
            h = ph - CR_SIZE - SH_SIZE - 4 - SH_SIZE # For Closer
        return (x,y,w,h)

    def OnSize(self,evt):
        x,y,w,h = self.CalcSizePos(self.GetParent())
        self.SetSize(x,y,w,h)

    def OnLeave(self,evt):
        self.SetCursor(wx.Cursor(wx.CURSOR_ARROW))

    def OnEnter(self,evt):
        if not self.GetParent().CanSize(not self.side):
            return
        if self.side == MV_HOR:
            self.SetCursor(wx.Cursor(wx.CURSOR_SIZENS))
        else:
            self.SetCursor(wx.Cursor(wx.CURSOR_SIZEWE))

    def OnMouseMove(self,evt):
        if self.isDrag:
            DrawSash(self.dragTarget,self.px,self.py,self.side)
            self.px,self.py = self.ClientToScreen((evt.x, evt.y))
            self.px,self.py = self.dragTarget.ScreenToClient((self.px,self.py))
            DrawSash(self.dragTarget,self.px,self.py,self.side)
        else:
            evt.Skip()

    def OnPress(self,evt):
        self.dragTarget = self.GetParent().SizeTarget(not self.side)
        if self.dragTarget:
            self.isDrag = True
            self.px,self.py = self.ClientToScreen((evt.x, evt.y))
            self.px,self.py = self.dragTarget.ScreenToClient((self.px,self.py))
            DrawSash(self.dragTarget,self.px,self.py,self.side)
            self.CaptureMouse()
        else:
            evt.Skip()

    def OnRelease(self,evt):
        if self.isDrag:
            DrawSash(self.dragTarget,self.px,self.py,self.side)
            self.ReleaseMouse()
            self.isDrag = False
            if self.side == MV_HOR:
                self.dragTarget.SizeLeaf(self.GetParent(),
                                         self.py,not self.side)
            else:
                self.dragTarget.SizeLeaf(self.GetParent(),
                                         self.px,not self.side)
            self.dragTarget = None
        else:
            evt.Skip()

#----------------------------------------------------------------------


class MultiCreator(wx.Window):
    def __init__(self,parent,side):
        self.side = side
        x,y,w,h = self.CalcSizePos(parent)
        wx.Window.__init__(self,id = -1,parent = parent,
                          pos = (x,y),
                          size = (w,h),
                          style = wx.CLIP_CHILDREN)

        self.px = None                  # Previous X
        self.py = None                  # Previous Y
        self.isDrag = False           # In Dragging

        self.Bind(wx.EVT_LEAVE_WINDOW,self.OnLeave)
        self.Bind(wx.EVT_ENTER_WINDOW,self.OnEnter)
        self.Bind(wx.EVT_MOTION,self.OnMouseMove)
        self.Bind(wx.EVT_LEFT_DOWN,self.OnPress)
        self.Bind(wx.EVT_LEFT_UP,self.OnRelease)
        self.Bind(wx.EVT_PAINT,self.OnPaint)

    def CalcSizePos(self,parent):
        pw,ph = parent.GetSize()
        if self.side == MV_HOR:
            x = 2
            y = ph - SH_SIZE
            w = CR_SIZE
            h = SH_SIZE
        else:
            x = pw - SH_SIZE
            y = 4 + SH_SIZE             # Make provision for closer
            w = SH_SIZE
            h = CR_SIZE
        return (x,y,w,h)

    def OnSize(self,evt):
        x,y,w,h = self.CalcSizePos(self.GetParent())
        self.SetSize(x,y,w,h)

    def OnLeave(self,evt):
        self.SetCursor(wx.Cursor(wx.CURSOR_ARROW))

    def OnEnter(self,evt):
        if self.side == MV_HOR:
            self.SetCursor(wx.Cursor(wx.CURSOR_HAND))
        else:
            self.SetCursor(wx.Cursor(wx.CURSOR_POINT_LEFT))

    def OnMouseMove(self,evt):
        if self.isDrag:
            parent = self.GetParent()
            DrawSash(parent,self.px,self.py,self.side)
            self.px,self.py = self.ClientToScreen((evt.x, evt.y))
            self.px,self.py = parent.ScreenToClient((self.px,self.py))
            DrawSash(parent,self.px,self.py,self.side)
        else:
            evt.Skip()

    def OnPress(self,evt):
        self.isDrag = True
        parent = self.GetParent()
        self.px,self.py = self.ClientToScreen((evt.x, evt.y))
        self.px,self.py = parent.ScreenToClient((self.px,self.py))
        DrawSash(parent,self.px,self.py,self.side)
        self.CaptureMouse()

    def OnRelease(self,evt):
        if self.isDrag:
            parent = self.GetParent()
            DrawSash(parent,self.px,self.py,self.side)
            self.ReleaseMouse()
            self.isDrag = False

            if self.side == MV_HOR:
                parent.AddLeaf(MV_VER,self.py)
            else:
                parent.AddLeaf(MV_HOR,self.px)
        else:
            evt.Skip()

    def OnPaint(self,evt):
        dc = wx.PaintDC(self)
        dc.SetBackground(wx.Brush(self.GetBackgroundColour(),wx.BRUSHSTYLE_SOLID))
        dc.Clear()

        highlight = wx.Pen(wx.SystemSettings.GetColour(wx.SYS_COLOUR_BTNHIGHLIGHT), 1, wx.PENSTYLE_SOLID)
        shadow = wx.Pen(wx.SystemSettings.GetColour(wx.SYS_COLOUR_BTNSHADOW), 1, wx.PENSTYLE_SOLID)
        black = wx.Pen(wx.BLACK,1,wx.PENSTYLE_SOLID)
        w,h = self.GetSize()
        w -= 1
        h -= 1

        # Draw outline
        dc.SetPen(highlight)
        dc.DrawLine(0,0, 0,h)
        dc.DrawLine(0,0, w,0)
        dc.SetPen(black)
        dc.DrawLine(0,h, w+1,h)
        dc.DrawLine(w,0, w,h)
        dc.SetPen(shadow)
        dc.DrawLine(w-1,2, w-1,h)

#----------------------------------------------------------------------


class MultiCloser(wx.Window):
    def __init__(self,parent):
        x,y,w,h = self.CalcSizePos(parent)
        wx.Window.__init__(self,id = -1,parent = parent,
                          pos = (x,y),
                          size = (w,h),
                          style = wx.CLIP_CHILDREN)

        self.down = False
        self.entered = False

        self.Bind(wx.EVT_LEFT_DOWN,self.OnPress)
        self.Bind(wx.EVT_LEFT_UP,self.OnRelease)
        self.Bind(wx.EVT_PAINT,self.OnPaint)
        self.Bind(wx.EVT_LEAVE_WINDOW,self.OnLeave)
        self.Bind(wx.EVT_ENTER_WINDOW,self.OnEnter)

    def OnLeave(self,evt):
        self.SetCursor(wx.Cursor(wx.CURSOR_ARROW))
        self.entered = False

    def OnEnter(self,evt):
        self.SetCursor(wx.Cursor(wx.CURSOR_BULLSEYE))
        self.entered = True

    def OnPress(self,evt):
        self.down = True
        evt.Skip()

    def OnRelease(self,evt):
        if self.down and self.entered:
            self.GetParent().DestroyLeaf()
        else:
            evt.Skip()
        self.down = False

    def OnPaint(self,evt):
        dc = wx.PaintDC(self)
        dc.SetBackground(wx.Brush(wx.RED,wx.BRUSHSTYLE_SOLID))
        dc.Clear()

    def CalcSizePos(self,parent):
        pw,ph = parent.GetSize()
        x = pw - SH_SIZE
        w = SH_SIZE
        h = SH_SIZE + 2
        y = 1
        return (x,y,w,h)

    def OnSize(self,evt):
        x,y,w,h = self.CalcSizePos(self.GetParent())
        self.SetSize(x,y,w,h)


#----------------------------------------------------------------------


class EmptyChild(wx.Window):
    def __init__(self,parent):
        wx.Window.__init__(self,parent,-1, style = wx.CLIP_CHILDREN)


#----------------------------------------------------------------------

# TODO: Switch to wx.Overlay instead of screen DC

def DrawSash(win,x,y,direction):
    dc = wx.ScreenDC()
    dc.StartDrawingOnTop(win)
    bmp = wx.Bitmap(8,8)
    bdc = wx.MemoryDC()
    bdc.SelectObject(bmp)
    bdc.DrawRectangle(-1,-1, 10,10)
    for i in range(8):
        for j in range(8):
            if ((i + j) & 1):
                bdc.DrawPoint(i,j)

    brush = wx.Brush(wx.Colour(0,0,0))
    brush.SetStipple(bmp)

    dc.SetBrush(brush)
    dc.SetLogicalFunction(wx.XOR)

    body_w,body_h = win.GetClientSize()

    if y < 0:
        y = 0
    if y > body_h:
        y = body_h
    if x < 0:
        x = 0
    if x > body_w:
        x = body_w

    if direction == MV_HOR:
        x = 0
    else:
        y = 0

    x,y = win.ClientToScreen((x,y))

    w = body_w
    h = body_h

    if direction == MV_HOR:
        dc.DrawRectangle(x,y-2, w,4)
    else:
        dc.DrawRectangle(x-2,y, 4,h)

    dc.EndDrawingOnTop()



#----------------------------------------------------------------------
# Name:        wx.lib.statbmp
# Purpose:     A generic StaticBitmap class.
#
# Author:      Robin Dunn
#
# Created:     12-May-2004
# Copyright:   (c) 2004-2018 by Total Control Software
# Licence:     wxWindows license
# Tags:        phoenix-port, unittest, documented, py3-port
#----------------------------------------------------------------------

"""
:class:`GenStaticBitmap` is a generic implementation of :class:`wx.StaticBitmap`.


Description
===========

:class:`GenStaticBitmap` is a generic implementation of
:class:`wx.StaticBitmap`.

Some of the platforms supported by wxPython (most notably GTK), do not
consider :class:`wx.StaticBitmap` as a separate widget; instead, the bitmap is
just drawn on its parent window. This essentially bars the use of almost all
mouse events (such as detection of mouse motions, mouse clicks and so on) on
the widget.

Using :class:`GenStaticBitmap` will overcome the problems described above, as
it is a generic widget and a real window on its own.


Usage
=====

Sample usage::

    import wx
    import wx.lib.statbmp as SB

    app = wx.App(0)

    frame = wx.Frame(None, -1, "wx.lib.statbmp Test")
    panel = wx.Panel(frame)

    bmp = wx.ArtProvider.GetBitmap(wx.ART_INFORMATION, wx.ART_OTHER, (16, 16))
    st1 = SB.GenStaticBitmap(panel, -1, bmp, (20, 10))

    bmp = wx.ArtProvider.GetBitmap(wx.ART_WARNING, wx.ART_OTHER, (32, 32))
    st2 = SB.GenStaticBitmap(panel, -1, bmp, (20, 60))

    frame.Show()
    app.MainLoop()


"""

import wx

#----------------------------------------------------------------------

class GenStaticBitmap(wx.Control):
    """ :class:`GenStaticBitmap` is a generic implementation of :class:`wx.StaticBitmap`. """

    labelDelta = 1

    def __init__(self, parent, ID, bitmap,
                 pos = wx.DefaultPosition, size = wx.DefaultSize,
                 style = 0,
                 name = "genstatbmp"):
        """
        Default class constructor.

        :param `parent`: parent window, must not be ``None``;
        :param integer `ID`: window identifier. A value of -1 indicates a default value;
        :param wx.Bitmap `bitmap`: the static bitmap used in the control;
        :param `pos`: the control position. A value of (-1, -1) indicates a default position,
         chosen by either the windowing system or wxPython, depending on platform;
        :param `size`: the control size. A value of (-1, -1) indicates a default size,
         chosen by either the windowing system or wxPython, depending on platform;
        :param integer `style`: the underlying :class:`wx.Control` style;
        :param string `name`: the widget name.

        :type parent: :class:`wx.Window`
        :type pos: tuple or :class:`wx.Point`
        :type size: tuple or :class:`wx.Size`
        """

        if not style & wx.BORDER_MASK:
            style = style | wx.BORDER_NONE

        wx.Control.__init__(self, parent, ID, pos, size, style,
                             wx.DefaultValidator, name)
        self._bitmap = bitmap
        self.InheritAttributes()
        self.SetInitialSize(size)

        self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)
        self.Bind(wx.EVT_PAINT,            self.OnPaint)


    def SetBitmap(self, bitmap):
        """
        Sets the bitmap label.

        :param wx.Bitmap `bitmap`: the new bitmap.

        .. seealso:: :meth:`GetBitmap`
        """

        self._bitmap = bitmap
        self.SetInitialSize( (bitmap.GetWidth(), bitmap.GetHeight()) )
        self.Refresh()


    def GetBitmap(self):
        """
        Returns the bitmap currently used in the control.

        :rtype: wx.Bitmap

        .. seealso:: :meth:`SetBitmap`
        """

        return self._bitmap


    def DoGetBestSize(self):
        """
        Overridden base class virtual.  Determines the best size of
        the control based on the label size and the current font.

        .. note:: Overridden from :class:`wx.Control`.
        """

        return wx.Size(self._bitmap.GetWidth(), self._bitmap.GetHeight())


    def AcceptsFocus(self):
        """
        Can this window be given focus by mouse click?

        .. note:: Overridden from :class:`wx.Control`.
        """

        return False


    def GetDefaultAttributes(self):
        """
        Overridden base class virtual.  By default we should use
        the same font/colour attributes as the native :class:`StaticBitmap`.

        .. note:: Overridden from :class:`wx.Control`.
        """

        return wx.StaticBitmap.GetClassDefaultAttributes()


    def ShouldInheritColours(self):
        """
        Overridden base class virtual.  If the parent has non-default
        colours then we want this control to inherit them.

        .. note:: Overridden from :class:`wx.Control`.
        """

        return True


    def OnPaint(self, event):
        """
        Handles the ``wx.EVT_PAINT`` for :class:`GenStaticBitmap`.

        :param `event`: a :class:`wx.PaintEvent` event to be processed.
        """

        dc = wx.PaintDC(self)
        if self._bitmap:
            dc.DrawBitmap(self._bitmap, 0, 0, True)


    def OnEraseBackground(self, event):
        """
        Handles the ``wx.EVT_ERASE_BACKGROUND`` event for :class:`GenStaticBitmap`.

        :param `event`: a :class:`wx.EraseEvent` event to be processed.

        .. note:: This is intentionally empty to reduce flicker.
        """

        pass




#----------------------------------------------------------------------





# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# This file is generated by wxPython's PI generator.  Do not edit by hand.
#
# The *.pyi files are used by PyCharm and other development tools to provide
# more information, such as PEP 484 type hints, than it is able to glean from
# introspection of extension types and methods.  They are not intended to be
# imported, executed or used for any other purpose other than providing info
# to the tools. If you don't use use a tool that makes use of .pyi files then
# you can safely ignore this file.
#
# See: https://www.python.org/dev/peps/pep-0484/
#      https://www.jetbrains.com/help/pycharm/2016.1/type-hinting-in-pycharm.html
#
# Copyright: (c) 2018 by Total Control Software
# License:   wxWindows License
#---------------------------------------------------------------------------


"""
The Grid and related classes in this module provide functionality similar to a
spreadsheet, where the application can display rows and columns of data of
various types, which the user can edit and otherwise interact with.
"""
#-- begin-_grid --#

import wx
#-- end-_grid --#
#-- begin-grid --#
GridNameStr = ""
GRID_AUTOSIZE = 0
GRID_COLUMN = 0
GRID_ROW = 0
GRID_FLOAT_FORMAT_FIXED = 0
GRID_FLOAT_FORMAT_SCIENTIFIC = 0
GRID_FLOAT_FORMAT_COMPACT = 0
GRID_FLOAT_FORMAT_UPPER = 0
GRID_FLOAT_FORMAT_DEFAULT = 0
GRIDTABLE_REQUEST_VIEW_GET_VALUES = 0
GRIDTABLE_REQUEST_VIEW_SEND_VALUES = 0
GRIDTABLE_NOTIFY_ROWS_INSERTED = 0
GRIDTABLE_NOTIFY_ROWS_APPENDED = 0
GRIDTABLE_NOTIFY_ROWS_DELETED = 0
GRIDTABLE_NOTIFY_COLS_INSERTED = 0
GRIDTABLE_NOTIFY_COLS_APPENDED = 0
GRIDTABLE_NOTIFY_COLS_DELETED = 0
GRID_DRAW_ROWS_HEADER = 0
GRID_DRAW_COLS_HEADER = 0
GRID_DRAW_CELL_LINES = 0
GRID_DRAW_BOX_RECT = 0
GRID_DRAW_SELECTION = 0
GRID_DRAW_DEFAULT = 0
wxEVT_GRID_CELL_LEFT_CLICK = 0
wxEVT_GRID_CELL_RIGHT_CLICK = 0
wxEVT_GRID_CELL_LEFT_DCLICK = 0
wxEVT_GRID_CELL_RIGHT_DCLICK = 0
wxEVT_GRID_LABEL_LEFT_CLICK = 0
wxEVT_GRID_LABEL_RIGHT_CLICK = 0
wxEVT_GRID_LABEL_LEFT_DCLICK = 0
wxEVT_GRID_LABEL_RIGHT_DCLICK = 0
wxEVT_GRID_ROW_SIZE = 0
wxEVT_GRID_COL_SIZE = 0
wxEVT_GRID_COL_AUTO_SIZE = 0
wxEVT_GRID_RANGE_SELECT = 0
wxEVT_GRID_CELL_CHANGING = 0
wxEVT_GRID_CELL_CHANGED = 0
wxEVT_GRID_SELECT_CELL = 0
wxEVT_GRID_EDITOR_SHOWN = 0
wxEVT_GRID_EDITOR_HIDDEN = 0
wxEVT_GRID_EDITOR_CREATED = 0
wxEVT_GRID_CELL_BEGIN_DRAG = 0
wxEVT_GRID_COL_MOVE = 0
wxEVT_GRID_COL_SORT = 0
wxEVT_GRID_TABBING = 0

class GridCellCoords(object):
    """
    GridCellCoords()
    GridCellCoords(row, col)
    
    Represents coordinates of a grid cell.
    """

    def __init__(self, *args, **kw):
        """
        GridCellCoords()
        GridCellCoords(row, col)
        
        Represents coordinates of a grid cell.
        """

    def GetRow(self):
        """
        GetRow() -> int
        
        Return the row of the coordinate.
        """

    def SetRow(self, n):
        """
        SetRow(n)
        
        Set the row of the coordinate.
        """

    def GetCol(self):
        """
        GetCol() -> int
        
        Return the column of the coordinate.
        """

    def SetCol(self, n):
        """
        SetCol(n)
        
        Set the column of the coordinate.
        """

    def Set(self, row, col):
        """
        Set(row, col)
        
        Set the row and column of the coordinate.
        """

    def __eq__(self):
        """
        """

    def __ne__(self):
        """
        """
    Col = property(None, None)
    Row = property(None, None)

    def Get(self):
        """
        Get() -> (row,col)
        
        Return the row and col properties as a tuple.
        """

    def GetIM(self):
        """
        Returns an immutable representation of the ``wx.GridCellCoords`` object, based on ``namedtuple``.
        
        This new object is hashable and can be used as a dictionary key,
        be added to sets, etc.  It can be converted back into a real ``wx.GridCellCoords``
        with a simple statement like this: ``obj = wx.GridCellCoords(imObj)``.
        """

    def __str__(self):
        """
        
        """

    def __repr__(self):
        """
        
        """

    def __len__(self):
        """
        
        """

    def __nonzero__(self):
        """
        
        """

    def __reduce__(self):
        """
        
        """

    def __getitem__(self, idx):
        """
        
        """

    def __setitem__(self, idx, val):
        """
        
        """

    __safe_for_unpickling__ = True
# end of class GridCellCoords


class GridCellRenderer(wx.ClientDataContainer, wx.RefCounter):
    """
    GridCellRenderer()
    
    This class is responsible for actually drawing the cell in the grid.
    """

    def __init__(self):
        """
        GridCellRenderer()
        
        This class is responsible for actually drawing the cell in the grid.
        """

    def Clone(self):
        """
        Clone() -> GridCellRenderer
        
        This function must be implemented in derived classes to return a copy
        of itself.
        """

    def Draw(self, grid, attr, dc, rect, row, col, isSelected):
        """
        Draw(grid, attr, dc, rect, row, col, isSelected)
        
        Draw the given cell on the provided DC inside the given rectangle
        using the style specified by the attribute and the default or selected
        state corresponding to the isSelected value.
        """

    def GetBestSize(self, grid, attr, dc, row, col):
        """
        GetBestSize(grid, attr, dc, row, col) -> wx.Size
        
        Get the preferred size of the cell for its contents.
        """
# end of class GridCellRenderer


class GridCellStringRenderer(GridCellRenderer):
    """
    GridCellStringRenderer()
    
    This class may be used to format string data in a cell; it is the
    default for string cells.
    """

    def __init__(self):
        """
        GridCellStringRenderer()
        
        This class may be used to format string data in a cell; it is the
        default for string cells.
        """
# end of class GridCellStringRenderer


class GridCellAutoWrapStringRenderer(GridCellStringRenderer):
    """
    GridCellAutoWrapStringRenderer()
    
    This class may be used to format string data in a cell.
    """

    def __init__(self):
        """
        GridCellAutoWrapStringRenderer()
        
        This class may be used to format string data in a cell.
        """
# end of class GridCellAutoWrapStringRenderer


class GridCellBoolRenderer(GridCellRenderer):
    """
    GridCellBoolRenderer()
    
    This class may be used to format boolean data in a cell.
    """

    def __init__(self):
        """
        GridCellBoolRenderer()
        
        This class may be used to format boolean data in a cell.
        """
# end of class GridCellBoolRenderer


class GridCellDateTimeRenderer(GridCellStringRenderer):
    """
    GridCellDateTimeRenderer(outformat=wx.DefaultDateTimeFormat, informat=wx.DefaultDateTimeFormat)
    
    This class may be used to format a date/time data in a cell.
    """

    def __init__(self, outformat=wx.DefaultDateTimeFormat, informat=wx.DefaultDateTimeFormat):
        """
        GridCellDateTimeRenderer(outformat=wx.DefaultDateTimeFormat, informat=wx.DefaultDateTimeFormat)
        
        This class may be used to format a date/time data in a cell.
        """

    def SetParameters(self, params):
        """
        SetParameters(params)
        
        Sets the strptime()-like format string which will be used to parse the
        date/time.
        """
# end of class GridCellDateTimeRenderer


class GridCellEnumRenderer(GridCellStringRenderer):
    """
    GridCellEnumRenderer(choices=wx.EmptyString)
    
    This class may be used to render in a cell a number as a textual
    equivalent.
    """

    def __init__(self, choices=wx.EmptyString):
        """
        GridCellEnumRenderer(choices=wx.EmptyString)
        
        This class may be used to render in a cell a number as a textual
        equivalent.
        """

    def SetParameters(self, params):
        """
        SetParameters(params)
        
        Sets the comma separated string content of the enum.
        """
# end of class GridCellEnumRenderer


class GridCellFloatRenderer(GridCellStringRenderer):
    """
    GridCellFloatRenderer(width=-1, precision=-1, format=GRID_FLOAT_FORMAT_DEFAULT)
    
    This class may be used to format floating point data in a cell.
    """

    def __init__(self, width=-1, precision=-1, format=GRID_FLOAT_FORMAT_DEFAULT):
        """
        GridCellFloatRenderer(width=-1, precision=-1, format=GRID_FLOAT_FORMAT_DEFAULT)
        
        This class may be used to format floating point data in a cell.
        """

    def GetFormat(self):
        """
        GetFormat() -> int
        
        Returns the specifier used to format the data to string.
        """

    def GetPrecision(self):
        """
        GetPrecision() -> int
        
        Returns the precision.
        """

    def GetWidth(self):
        """
        GetWidth() -> int
        
        Returns the width.
        """

    def SetFormat(self, format):
        """
        SetFormat(format)
        
        Set the format to use for display the number.
        """

    def SetParameters(self, params):
        """
        SetParameters(params)
        
        The parameters string format is "width[,precision[,format]]" where
        format should be chosen between f|e|g|E|G (f is used by default)
        """

    def SetPrecision(self, precision):
        """
        SetPrecision(precision)
        
        Sets the precision.
        """

    def SetWidth(self, width):
        """
        SetWidth(width)
        
        Sets the width.
        """
    Format = property(None, None)
    Precision = property(None, None)
    Width = property(None, None)
# end of class GridCellFloatRenderer


class GridCellNumberRenderer(GridCellStringRenderer):
    """
    GridCellNumberRenderer()
    
    This class may be used to format integer data in a cell.
    """

    def __init__(self):
        """
        GridCellNumberRenderer()
        
        This class may be used to format integer data in a cell.
        """
# end of class GridCellNumberRenderer


class GridCellEditor(wx.ClientDataContainer, wx.RefCounter):
    """
    GridCellEditor()
    
    This class is responsible for providing and manipulating the in-place
    edit controls for the grid.
    """

    def __init__(self):
        """
        GridCellEditor()
        
        This class is responsible for providing and manipulating the in-place
        edit controls for the grid.
        """

    def BeginEdit(self, row, col, grid):
        """
        BeginEdit(row, col, grid)
        
        Fetch the value from the table and prepare the edit control to begin
        editing.
        """

    def Clone(self):
        """
        Clone() -> GridCellEditor
        
        Create a new object which is the copy of this one.
        """

    def Create(self, parent, id, evtHandler):
        """
        Create(parent, id, evtHandler)
        
        Creates the actual edit control.
        """

    def Destroy(self):
        """
        Destroy()
        
        Final cleanup.
        """

    def EndEdit(self, row, col, grid, oldval):
        """
        EndEdit(row, col, grid, oldval)
        
        End editing the cell.
        
        This function must check if the current value of the editing cell
        is valid and different from the original value in its string
        form. If not then simply return None.  If it has changed then
        this method should save the new value so that ApplyEdit can
        apply it later and the string representation of the new value
        should be returned.
        
        Notice that this method shoiuld not modify the grid as the
        change could still be vetoed.
        """

    def ApplyEdit(self, row, col, grid):
        """
        ApplyEdit(row, col, grid)
        
        Effectively save the changes in the grid.
        """

    def HandleReturn(self, event):
        """
        HandleReturn(event)
        
        Some types of controls on some platforms may need some help with the
        Return key.
        """

    def IsCreated(self):
        """
        IsCreated() -> bool
        
        Returns true if the edit control has been created.
        """

    def PaintBackground(self, dc, rectCell, attr):
        """
        PaintBackground(dc, rectCell, attr)
        
        Draws the part of the cell not occupied by the control: the base class
        version just fills it with background colour from the attribute.
        """

    def Reset(self):
        """
        Reset()
        
        Reset the value in the control back to its starting value.
        """

    def SetSize(self, rect):
        """
        SetSize(rect)
        
        Size and position the edit control.
        """

    def Show(self, show, attr=None):
        """
        Show(show, attr=None)
        
        Show or hide the edit control, use the specified attributes to set
        colours/fonts for it.
        """

    def StartingClick(self):
        """
        StartingClick()
        
        If the editor is enabled by clicking on the cell, this method will be
        called.
        """

    def StartingKey(self, event):
        """
        StartingKey(event)
        
        If the editor is enabled by pressing keys on the grid, this will be
        called to let the editor do something about that first key if desired.
        """

    def GetValue(self):
        """
        GetValue() -> String
        
        Returns the value currently in the editor control.
        """

    def GetControl(self):
        """
        GetControl() -> wx.Control
        
        Get the wxControl used by this editor.
        """

    def SetControl(self, control):
        """
        SetControl(control)
        
        Set the wxControl that will be used by this cell editor for editing
        the value.
        """
    Control = property(None, None)
    Value = property(None, None)
# end of class GridCellEditor


class GridCellTextEditor(GridCellEditor):
    """
    GridCellTextEditor(maxChars=0)
    
    Grid cell editor for string/text data.
    """

    def __init__(self, maxChars=0):
        """
        GridCellTextEditor(maxChars=0)
        
        Grid cell editor for string/text data.
        """

    def SetParameters(self, params):
        """
        SetParameters(params)
        
        The parameters string format is "n" where n is a number representing
        the maximum width.
        """

    def SetValidator(self, validator):
        """
        SetValidator(validator)
        
        Set validator to validate user input.
        """

    def EndEdit(self, row, col, grid, oldval):
        """
        EndEdit(row, col, grid, oldval)
        
        End editing the cell.
        
        This function must check if the current value of the editing cell
        is valid and different from the original value in its string
        form. If not then simply return None.  If it has changed then
        this method should save the new value so that ApplyEdit can
        apply it later and the string representation of the new value
        should be returned.
        
        Notice that this method shoiuld not modify the grid as the
        change could still be vetoed.
        """
# end of class GridCellTextEditor


class GridCellAutoWrapStringEditor(GridCellTextEditor):
    """
    GridCellAutoWrapStringEditor()
    
    Grid cell editor for wrappable string/text data.
    """

    def __init__(self):
        """
        GridCellAutoWrapStringEditor()
        
        Grid cell editor for wrappable string/text data.
        """

    def EndEdit(self, row, col, grid, oldval):
        """
        EndEdit(row, col, grid, oldval)
        
        End editing the cell.
        
        This function must check if the current value of the editing cell
        is valid and different from the original value in its string
        form. If not then simply return None.  If it has changed then
        this method should save the new value so that ApplyEdit can
        apply it later and the string representation of the new value
        should be returned.
        
        Notice that this method shoiuld not modify the grid as the
        change could still be vetoed.
        """
# end of class GridCellAutoWrapStringEditor


class GridCellBoolEditor(GridCellEditor):
    """
    GridCellBoolEditor()
    
    Grid cell editor for boolean data.
    """

    def __init__(self):
        """
        GridCellBoolEditor()
        
        Grid cell editor for boolean data.
        """

    @staticmethod
    def IsTrueValue(value):
        """
        IsTrueValue(value) -> bool
        
        Returns true if the given value is equal to the string representation
        of the truth value we currently use (see UseStringValues()).
        """

    @staticmethod
    def UseStringValues(valueTrue="1", valueFalse=wx.EmptyString):
        """
        UseStringValues(valueTrue="1", valueFalse=wx.EmptyString)
        
        This method allows you to customize the values returned by GetValue()
        for the cell using this editor.
        """

    def EndEdit(self, row, col, grid, oldval):
        """
        EndEdit(row, col, grid, oldval)
        
        End editing the cell.
        
        This function must check if the current value of the editing cell
        is valid and different from the original value in its string
        form. If not then simply return None.  If it has changed then
        this method should save the new value so that ApplyEdit can
        apply it later and the string representation of the new value
        should be returned.
        
        Notice that this method shoiuld not modify the grid as the
        change could still be vetoed.
        """
# end of class GridCellBoolEditor


class GridCellChoiceEditor(GridCellEditor):
    """
    GridCellChoiceEditor(choices, allowOthers=False)
    
    Grid cell editor for string data providing the user a choice from a
    list of strings.
    """

    def __init__(self, choices, allowOthers=False):
        """
        GridCellChoiceEditor(choices, allowOthers=False)
        
        Grid cell editor for string data providing the user a choice from a
        list of strings.
        """

    def SetParameters(self, params):
        """
        SetParameters(params)
        
        Parameters string format is "item1[,item2[...,itemN]]".
        """

    def EndEdit(self, row, col, grid, oldval):
        """
        EndEdit(row, col, grid, oldval)
        
        End editing the cell.
        
        This function must check if the current value of the editing cell
        is valid and different from the original value in its string
        form. If not then simply return None.  If it has changed then
        this method should save the new value so that ApplyEdit can
        apply it later and the string representation of the new value
        should be returned.
        
        Notice that this method shoiuld not modify the grid as the
        change could still be vetoed.
        """
# end of class GridCellChoiceEditor


class GridCellEnumEditor(GridCellChoiceEditor):
    """
    GridCellEnumEditor(choices=wx.EmptyString)
    
    Grid cell editor which displays an enum number as a textual equivalent
    (eg.
    """

    def __init__(self, choices=wx.EmptyString):
        """
        GridCellEnumEditor(choices=wx.EmptyString)
        
        Grid cell editor which displays an enum number as a textual equivalent
        (eg.
        """

    def EndEdit(self, row, col, grid, oldval):
        """
        EndEdit(row, col, grid, oldval)
        
        End editing the cell.
        
        This function must check if the current value of the editing cell
        is valid and different from the original value in its string
        form. If not then simply return None.  If it has changed then
        this method should save the new value so that ApplyEdit can
        apply it later and the string representation of the new value
        should be returned.
        
        Notice that this method shoiuld not modify the grid as the
        change could still be vetoed.
        """
# end of class GridCellEnumEditor


class GridCellFloatEditor(GridCellTextEditor):
    """
    GridCellFloatEditor(width=-1, precision=-1, format=GRID_FLOAT_FORMAT_DEFAULT)
    
    The editor for floating point numbers data.
    """

    def __init__(self, width=-1, precision=-1, format=GRID_FLOAT_FORMAT_DEFAULT):
        """
        GridCellFloatEditor(width=-1, precision=-1, format=GRID_FLOAT_FORMAT_DEFAULT)
        
        The editor for floating point numbers data.
        """

    def SetParameters(self, params):
        """
        SetParameters(params)
        
        The parameters string format is "width[,precision[,format]]" where
        format should be chosen between f|e|g|E|G (f is used by default)
        """

    def EndEdit(self, row, col, grid, oldval):
        """
        EndEdit(row, col, grid, oldval)
        
        End editing the cell.
        
        This function must check if the current value of the editing cell
        is valid and different from the original value in its string
        form. If not then simply return None.  If it has changed then
        this method should save the new value so that ApplyEdit can
        apply it later and the string representation of the new value
        should be returned.
        
        Notice that this method shoiuld not modify the grid as the
        change could still be vetoed.
        """
# end of class GridCellFloatEditor


class GridCellNumberEditor(GridCellTextEditor):
    """
    GridCellNumberEditor(min=-1, max=-1)
    
    Grid cell editor for numeric integer data.
    """

    def __init__(self, min=-1, max=-1):
        """
        GridCellNumberEditor(min=-1, max=-1)
        
        Grid cell editor for numeric integer data.
        """

    def SetParameters(self, params):
        """
        SetParameters(params)
        
        Parameters string format is "min,max".
        """

    def EndEdit(self, row, col, grid, oldval):
        """
        EndEdit(row, col, grid, oldval)
        
        End editing the cell.
        
        This function must check if the current value of the editing cell
        is valid and different from the original value in its string
        form. If not then simply return None.  If it has changed then
        this method should save the new value so that ApplyEdit can
        apply it later and the string representation of the new value
        should be returned.
        
        Notice that this method shoiuld not modify the grid as the
        change could still be vetoed.
        """
# end of class GridCellNumberEditor


class GridCellAttr(wx.ClientDataContainer, wx.RefCounter):
    """
    GridCellAttr(attrDefault=None)
    GridCellAttr(colText, colBack, font, hAlign, vAlign)
    
    This class can be used to alter the cells' appearance in the grid by
    changing their attributes from the defaults.
    """
    Any = 0
    Cell = 0
    Row = 0
    Col = 0
    Default = 0
    Merged = 0

    def __init__(self, *args, **kw):
        """
        GridCellAttr(attrDefault=None)
        GridCellAttr(colText, colBack, font, hAlign, vAlign)
        
        This class can be used to alter the cells' appearance in the grid by
        changing their attributes from the defaults.
        """

    def Clone(self):
        """
        Clone() -> GridCellAttr
        
        Creates a new copy of this object.
        """

    def DecRef(self):
        """
        DecRef()
        
        This class is reference counted: it is created with ref count of 1, so
        calling DecRef() once will delete it.
        """

    def GetAlignment(self):
        """
        GetAlignment() -> (hAlign, vAlign)
        
        Get the alignment to use for the cell with the given attribute.
        """

    def GetBackgroundColour(self):
        """
        GetBackgroundColour() -> wx.Colour
        
        Returns the background colour.
        """

    def GetEditor(self, grid, row, col):
        """
        GetEditor(grid, row, col) -> GridCellEditor
        
        Returns the cell editor.
        """

    def GetFont(self):
        """
        GetFont() -> wx.Font
        
        Returns the font.
        """

    def GetNonDefaultAlignment(self):
        """
        GetNonDefaultAlignment() -> (hAlign, vAlign)
        
        Get the alignment defined by this attribute.
        """

    def GetRenderer(self, grid, row, col):
        """
        GetRenderer(grid, row, col) -> GridCellRenderer
        
        Returns the cell renderer.
        """

    def GetTextColour(self):
        """
        GetTextColour() -> wx.Colour
        
        Returns the text colour.
        """

    def HasAlignment(self):
        """
        HasAlignment() -> bool
        
        Returns true if this attribute has a valid alignment set.
        """

    def HasBackgroundColour(self):
        """
        HasBackgroundColour() -> bool
        
        Returns true if this attribute has a valid background colour set.
        """

    def HasEditor(self):
        """
        HasEditor() -> bool
        
        Returns true if this attribute has a valid cell editor set.
        """

    def HasFont(self):
        """
        HasFont() -> bool
        
        Returns true if this attribute has a valid font set.
        """

    def HasRenderer(self):
        """
        HasRenderer() -> bool
        
        Returns true if this attribute has a valid cell renderer set.
        """

    def HasTextColour(self):
        """
        HasTextColour() -> bool
        
        Returns true if this attribute has a valid text colour set.
        """

    def IncRef(self):
        """
        IncRef()
        
        This class is reference counted: it is created with ref count of 1, so
        calling DecRef() once will delete it.
        """

    def IsReadOnly(self):
        """
        IsReadOnly() -> bool
        
        Returns true if this cell is set as read-only.
        """

    def SetAlignment(self, hAlign, vAlign):
        """
        SetAlignment(hAlign, vAlign)
        
        Sets the alignment.
        """

    def SetBackgroundColour(self, colBack):
        """
        SetBackgroundColour(colBack)
        
        Sets the background colour.
        """

    def SetDefAttr(self, defAttr):
        """
        SetDefAttr(defAttr)
        """

    def SetEditor(self, editor):
        """
        SetEditor(editor)
        
        Sets the editor to be used with the cells with this attribute.
        """

    def SetFont(self, font):
        """
        SetFont(font)
        
        Sets the font.
        """

    def SetReadOnly(self, isReadOnly=True):
        """
        SetReadOnly(isReadOnly=True)
        
        Sets the cell as read-only.
        """

    def SetRenderer(self, renderer):
        """
        SetRenderer(renderer)
        
        Sets the renderer to be used for cells with this attribute.
        """

    def SetTextColour(self, colText):
        """
        SetTextColour(colText)
        
        Sets the text colour.
        """

    def MergeWith(self, mergefrom):
        """
        MergeWith(mergefrom)
        """

    def SetSize(self, num_rows, num_cols):
        """
        SetSize(num_rows, num_cols)
        """

    def SetOverflow(self, allow=True):
        """
        SetOverflow(allow=True)
        """

    def SetKind(self, kind):
        """
        SetKind(kind)
        """

    def HasReadWriteMode(self):
        """
        HasReadWriteMode() -> bool
        """

    def HasOverflowMode(self):
        """
        HasOverflowMode() -> bool
        """

    def HasSize(self):
        """
        HasSize() -> bool
        """

    def GetSize(self):
        """
        GetSize() -> (num_rows, num_cols)
        """

    def GetOverflow(self):
        """
        GetOverflow() -> bool
        """

    def GetKind(self):
        """
        GetKind() -> AttrKind
        """
    BackgroundColour = property(None, None)
    Font = property(None, None)
    Kind = property(None, None)
    Overflow = property(None, None)
    TextColour = property(None, None)
# end of class GridCellAttr


class GridCornerHeaderRenderer(object):
    """
    Base class for corner window renderer.
    """

    def DrawBorder(self, grid, dc, rect):
        """
        DrawBorder(grid, dc, rect)
        
        Called by the grid to draw the corner window border.
        """
# end of class GridCornerHeaderRenderer


class GridHeaderLabelsRenderer(GridCornerHeaderRenderer):
    """
    Common base class for row and column headers renderers.
    """

    def DrawLabel(self, grid, dc, value, rect, horizAlign, vertAlign, textOrientation):
        """
        DrawLabel(grid, dc, value, rect, horizAlign, vertAlign, textOrientation)
        
        Called by the grid to draw the specified label.
        """
# end of class GridHeaderLabelsRenderer


class GridRowHeaderRenderer(GridHeaderLabelsRenderer):
    """
    Base class for row headers renderer.
    """
# end of class GridRowHeaderRenderer


class GridColumnHeaderRenderer(GridHeaderLabelsRenderer):
    """
    Base class for column headers renderer.
    """
# end of class GridColumnHeaderRenderer


class GridRowHeaderRendererDefault(GridRowHeaderRenderer):
    """
    Default row header renderer.
    """

    def DrawBorder(self, grid, dc, rect):
        """
        DrawBorder(grid, dc, rect)
        
        Implement border drawing for the row labels.
        """
# end of class GridRowHeaderRendererDefault


class GridColumnHeaderRendererDefault(GridColumnHeaderRenderer):
    """
    Default column header renderer.
    """

    def DrawBorder(self, grid, dc, rect):
        """
        DrawBorder(grid, dc, rect)
        
        Implement border drawing for the column labels.
        """
# end of class GridColumnHeaderRendererDefault


class GridCornerHeaderRendererDefault(GridCornerHeaderRenderer):
    """
    Default corner window renderer.
    """

    def DrawBorder(self, grid, dc, rect):
        """
        DrawBorder(grid, dc, rect)
        
        Implement border drawing for the corner window.
        """
# end of class GridCornerHeaderRendererDefault


class GridCellAttrProvider(wx.ClientDataContainer):
    """
    GridCellAttrProvider()
    
    Class providing attributes to be used for the grid cells.
    """

    def __init__(self):
        """
        GridCellAttrProvider()
        
        Class providing attributes to be used for the grid cells.
        """

    def SetAttr(self, attr, row, col):
        """
        SetAttr(attr, row, col)
        
        Set attribute for the specified cell.
        """

    def SetRowAttr(self, attr, row):
        """
        SetRowAttr(attr, row)
        
        Set attribute for the specified row.
        """

    def SetColAttr(self, attr, col):
        """
        SetColAttr(attr, col)
        
        Set attribute for the specified column.
        """

    def GetColumnHeaderRenderer(self, col):
        """
        GetColumnHeaderRenderer(col) -> GridColumnHeaderRenderer
        
        Return the renderer used for drawing column headers.
        """

    def GetRowHeaderRenderer(self, row):
        """
        GetRowHeaderRenderer(row) -> GridRowHeaderRenderer
        
        Return the renderer used for drawing row headers.
        """

    def GetCornerRenderer(self):
        """
        GetCornerRenderer() -> GridCornerHeaderRenderer
        
        Return the renderer used for drawing the corner window.
        """

    def GetAttr(self, row, col, kind):
        """
        GetAttr(row, col, kind) -> GridCellAttr
        
        Get the attribute to use for the specified cell.
        """
    CornerRenderer = property(None, None)
# end of class GridCellAttrProvider


class GridTableBase(wx.Object):
    """
    GridTableBase()
    
    The almost abstract base class for grid tables.
    """

    def __init__(self):
        """
        GridTableBase()
        
        The almost abstract base class for grid tables.
        """

    def IsEmptyCell(self, row, col):
        """
        IsEmptyCell(row, col) -> bool
        
        May be overridden to implement testing for empty cells.
        """

    def IsEmpty(self, coords):
        """
        IsEmpty(coords) -> bool
        
        Same as IsEmptyCell() but taking wxGridCellCoords.
        """

    def GetValue(self, row, col):
        """
        GetValue(row, col) -> PyObject
        
        Must be overridden to implement accessing the table values as text.
        """

    def SetValue(self, row, col, value):
        """
        SetValue(row, col, value)
        
        Must be overridden to implement setting the table values as text.
        """

    def GetTypeName(self, row, col):
        """
        GetTypeName(row, col) -> String
        
        Returns the type of the value in the given cell.
        """

    def CanGetValueAs(self, row, col, typeName):
        """
        CanGetValueAs(row, col, typeName) -> bool
        
        Returns true if the value of the given cell can be accessed as if it
        were of the specified type.
        """

    def CanSetValueAs(self, row, col, typeName):
        """
        CanSetValueAs(row, col, typeName) -> bool
        
        Returns true if the value of the given cell can be set as if it were
        of the specified type.
        """

    def GetValueAsLong(self, row, col):
        """
        GetValueAsLong(row, col) -> long
        
        Returns the value of the given cell as a long.
        """

    def GetValueAsDouble(self, row, col):
        """
        GetValueAsDouble(row, col) -> double
        
        Returns the value of the given cell as a double.
        """

    def GetValueAsBool(self, row, col):
        """
        GetValueAsBool(row, col) -> bool
        
        Returns the value of the given cell as a boolean.
        """

    def SetValueAsLong(self, row, col, value):
        """
        SetValueAsLong(row, col, value)
        
        Sets the value of the given cell as a long.
        """

    def SetValueAsDouble(self, row, col, value):
        """
        SetValueAsDouble(row, col, value)
        
        Sets the value of the given cell as a double.
        """

    def SetValueAsBool(self, row, col, value):
        """
        SetValueAsBool(row, col, value)
        
        Sets the value of the given cell as a boolean.
        """

    def Clear(self):
        """
        Clear()
        
        Clear the table contents.
        """

    def InsertRows(self, pos=0, numRows=1):
        """
        InsertRows(pos=0, numRows=1) -> bool
        
        Insert additional rows into the table.
        """

    def AppendRows(self, numRows=1):
        """
        AppendRows(numRows=1) -> bool
        
        Append additional rows at the end of the table.
        """

    def DeleteRows(self, pos=0, numRows=1):
        """
        DeleteRows(pos=0, numRows=1) -> bool
        
        Delete rows from the table.
        """

    def InsertCols(self, pos=0, numCols=1):
        """
        InsertCols(pos=0, numCols=1) -> bool
        
        Exactly the same as InsertRows() but for columns.
        """

    def AppendCols(self, numCols=1):
        """
        AppendCols(numCols=1) -> bool
        
        Exactly the same as AppendRows() but for columns.
        """

    def DeleteCols(self, pos=0, numCols=1):
        """
        DeleteCols(pos=0, numCols=1) -> bool
        
        Exactly the same as DeleteRows() but for columns.
        """

    def GetRowLabelValue(self, row):
        """
        GetRowLabelValue(row) -> String
        
        Return the label of the specified row.
        """

    def GetColLabelValue(self, col):
        """
        GetColLabelValue(col) -> String
        
        Return the label of the specified column.
        """

    def SetRowLabelValue(self, row, label):
        """
        SetRowLabelValue(row, label)
        
        Set the given label for the specified row.
        """

    def SetColLabelValue(self, col, label):
        """
        SetColLabelValue(col, label)
        
        Exactly the same as SetRowLabelValue() but for columns.
        """

    def SetAttrProvider(self, attrProvider):
        """
        SetAttrProvider(attrProvider)
        
        Associate this attributes provider with the table.
        """

    def GetAttrProvider(self):
        """
        GetAttrProvider() -> GridCellAttrProvider
        
        Returns the attribute provider currently being used.
        """

    def GetAttr(self, row, col, kind):
        """
        GetAttr(row, col, kind) -> GridCellAttr
        
        Return the attribute for the given cell.
        """

    def SetAttr(self, attr, row, col):
        """
        SetAttr(attr, row, col)
        
        Set attribute of the specified cell.
        """

    def SetRowAttr(self, attr, row):
        """
        SetRowAttr(attr, row)
        
        Set attribute of the specified row.
        """

    def SetColAttr(self, attr, col):
        """
        SetColAttr(attr, col)
        
        Set attribute of the specified column.
        """

    def GetNumberRows(self):
        """
        GetNumberRows() -> int
        
        Must be overridden to return the number of rows in the table.
        """

    def GetNumberCols(self):
        """
        GetNumberCols() -> int
        
        Must be overridden to return the number of columns in the table.
        """

    def GetRowsCount(self):
        """
        GetRowsCount() -> int
        
        Return the number of rows in the table.
        """

    def GetColsCount(self):
        """
        GetColsCount() -> int
        
        Return the number of columns in the table.
        """

    def SetView(self, grid):
        """
        SetView(grid)
        
        Called by the grid when the table is associated with it.
        """

    def GetView(self):
        """
        GetView() -> Grid
        
        Returns the last grid passed to SetView().
        """

    def CanHaveAttributes(self):
        """
        CanHaveAttributes() -> bool
        
        Returns true if this table supports attributes or false otherwise.
        """

    def GetValueAsLong(self, row, col):
        """
        
        """

    def GetValueAsDouble(self, row, col):
        """
        
        """

    def GetValueAsBool(self, row, col):
        """
        
        """

    def SetValueAsLong(self, row, col, value):
        """
        
        """

    def SetValueAsDouble(self, row, col, value):
        """
        
        """

    def SetValueAsBool(self, row, col, value):
        """
        
        """
    AttrProvider = property(None, None)
    ColsCount = property(None, None)
    NumberCols = property(None, None)
    NumberRows = property(None, None)
    RowsCount = property(None, None)
    View = property(None, None)
# end of class GridTableBase


class GridTableMessage(object):
    """
    GridTableMessage()
    GridTableMessage(table, id, comInt1=-1, comInt2=-1)
    
    A simple class used to pass messages from the table to the grid.
    """

    def __init__(self, *args, **kw):
        """
        GridTableMessage()
        GridTableMessage(table, id, comInt1=-1, comInt2=-1)
        
        A simple class used to pass messages from the table to the grid.
        """

    def SetTableObject(self, table):
        """
        SetTableObject(table)
        """

    def GetTableObject(self):
        """
        GetTableObject() -> GridTableBase
        """

    def SetId(self, id):
        """
        SetId(id)
        """

    def GetId(self):
        """
        GetId() -> int
        """

    def SetCommandInt(self, comInt1):
        """
        SetCommandInt(comInt1)
        """

    def GetCommandInt(self):
        """
        GetCommandInt() -> int
        """

    def SetCommandInt2(self, comInt2):
        """
        SetCommandInt2(comInt2)
        """

    def GetCommandInt2(self):
        """
        GetCommandInt2() -> int
        """
    CommandInt = property(None, None)
    CommandInt2 = property(None, None)
    Id = property(None, None)
    TableObject = property(None, None)
# end of class GridTableMessage


class GridStringTable(GridTableBase):
    """
    GridStringTable()
    GridStringTable(numRows, numCols)
    
    Simplest type of data table for a grid for small tables of strings
    that are stored in memory.
    """

    def __init__(self, *args, **kw):
        """
        GridStringTable()
        GridStringTable(numRows, numCols)
        
        Simplest type of data table for a grid for small tables of strings
        that are stored in memory.
        """

    def GetNumberRows(self):
        """
        GetNumberRows() -> int
        
        Must be overridden to return the number of rows in the table.
        """

    def GetNumberCols(self):
        """
        GetNumberCols() -> int
        
        Must be overridden to return the number of columns in the table.
        """

    def GetValue(self, row, col):
        """
        GetValue(row, col) -> String
        
        Must be overridden to implement accessing the table values as text.
        """

    def SetValue(self, row, col, value):
        """
        SetValue(row, col, value)
        
        Must be overridden to implement setting the table values as text.
        """

    def Clear(self):
        """
        Clear()
        
        Clear the table contents.
        """

    def InsertRows(self, pos=0, numRows=1):
        """
        InsertRows(pos=0, numRows=1) -> bool
        
        Insert additional rows into the table.
        """

    def AppendRows(self, numRows=1):
        """
        AppendRows(numRows=1) -> bool
        
        Append additional rows at the end of the table.
        """

    def DeleteRows(self, pos=0, numRows=1):
        """
        DeleteRows(pos=0, numRows=1) -> bool
        
        Delete rows from the table.
        """

    def InsertCols(self, pos=0, numCols=1):
        """
        InsertCols(pos=0, numCols=1) -> bool
        
        Exactly the same as InsertRows() but for columns.
        """

    def AppendCols(self, numCols=1):
        """
        AppendCols(numCols=1) -> bool
        
        Exactly the same as AppendRows() but for columns.
        """

    def DeleteCols(self, pos=0, numCols=1):
        """
        DeleteCols(pos=0, numCols=1) -> bool
        
        Exactly the same as DeleteRows() but for columns.
        """

    def SetRowLabelValue(self, row, label):
        """
        SetRowLabelValue(row, label)
        
        Set the given label for the specified row.
        """

    def SetColLabelValue(self, col, label):
        """
        SetColLabelValue(col, label)
        
        Exactly the same as SetRowLabelValue() but for columns.
        """

    def GetRowLabelValue(self, row):
        """
        GetRowLabelValue(row) -> String
        
        Return the label of the specified row.
        """

    def GetColLabelValue(self, col):
        """
        GetColLabelValue(col) -> String
        
        Return the label of the specified column.
        """
    NumberCols = property(None, None)
    NumberRows = property(None, None)
# end of class GridStringTable


class GridSizesInfo(object):
    """
    GridSizesInfo()
    GridSizesInfo(defSize, allSizes)
    
    wxGridSizesInfo stores information about sizes of all wxGrid rows or
    columns.
    """

    def __init__(self, *args, **kw):
        """
        GridSizesInfo()
        GridSizesInfo(defSize, allSizes)
        
        wxGridSizesInfo stores information about sizes of all wxGrid rows or
        columns.
        """
    m_sizeDefault = property(None, None)

    def GetSize(self, pos):
        """
        GetSize(pos) -> int
        
        Get the element size.
        """
# end of class GridSizesInfo


class Grid(wx.ScrolledWindow):
    """
    Grid()
    Grid(parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize, style=wx.WANTS_CHARS, name=GridNameStr)
    
    wxGrid and its related classes are used for displaying and editing
    tabular data.
    """
    SelectCells = 0
    SelectRows = 0
    SelectColumns = 0
    SelectRowsOrColumns = 0
    CellSpan_Inside = 0
    CellSpan_None = 0
    CellSpan_Main = 0
    Tab_Stop = 0
    Tab_Wrap = 0
    Tab_Leave = 0

    def __init__(self, *args, **kw):
        """
        Grid()
        Grid(parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize, style=wx.WANTS_CHARS, name=GridNameStr)
        
        wxGrid and its related classes are used for displaying and editing
        tabular data.
        """

    def Create(self, parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize, style=wx.WANTS_CHARS, name=GridNameStr):
        """
        Create(parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize, style=wx.WANTS_CHARS, name=GridNameStr) -> bool
        
        Creates the grid window for an object initialized using the default
        constructor.
        """

    def CreateGrid(self, numRows, numCols, selmode=GridSelectCells):
        """
        CreateGrid(numRows, numCols, selmode=GridSelectCells) -> bool
        
        Creates a grid with the specified initial number of rows and columns.
        """

    def _SetTable(self, table, takeOwnership=False, selmode=GridSelectCells):
        """
        _SetTable(table, takeOwnership=False, selmode=GridSelectCells) -> bool
        
        Passes a pointer to a custom grid table to be used by the grid.
        """

    def ProcessTableMessage(self, msg):
        """
        ProcessTableMessage(msg) -> bool
        
        Receive and handle a message from the table.
        """

    def EnableGridLines(self, enable=True):
        """
        EnableGridLines(enable=True)
        
        Turns the drawing of grid lines on or off.
        """

    def GetColGridLinePen(self, col):
        """
        GetColGridLinePen(col) -> wx.Pen
        
        Returns the pen used for vertical grid lines.
        """

    def GetDefaultGridLinePen(self):
        """
        GetDefaultGridLinePen() -> wx.Pen
        
        Returns the pen used for grid lines.
        """

    def GetGridLineColour(self):
        """
        GetGridLineColour() -> wx.Colour
        
        Returns the colour used for grid lines.
        """

    def GetRowGridLinePen(self, row):
        """
        GetRowGridLinePen(row) -> wx.Pen
        
        Returns the pen used for horizontal grid lines.
        """

    def GridLinesEnabled(self):
        """
        GridLinesEnabled() -> bool
        
        Returns true if drawing of grid lines is turned on, false otherwise.
        """

    def SetGridLineColour(self, colour):
        """
        SetGridLineColour(colour)
        
        Sets the colour used to draw grid lines.
        """

    def GetColLabelAlignment(self):
        """
        GetColLabelAlignment() -> (horiz, vert)
        
        Sets the arguments to the current column label alignment values.
        """

    def GetColLabelTextOrientation(self):
        """
        GetColLabelTextOrientation() -> int
        
        Returns the orientation of the column labels (either wxHORIZONTAL or
        wxVERTICAL).
        """

    def GetColLabelValue(self, col):
        """
        GetColLabelValue(col) -> String
        
        Returns the specified column label.
        """

    def GetLabelBackgroundColour(self):
        """
        GetLabelBackgroundColour() -> wx.Colour
        
        Returns the colour used for the background of row and column labels.
        """

    def GetLabelFont(self):
        """
        GetLabelFont() -> wx.Font
        
        Returns the font used for row and column labels.
        """

    def GetLabelTextColour(self):
        """
        GetLabelTextColour() -> wx.Colour
        
        Returns the colour used for row and column label text.
        """

    def GetRowLabelAlignment(self):
        """
        GetRowLabelAlignment() -> (horiz, vert)
        
        Returns the alignment used for row labels.
        """

    def GetRowLabelValue(self, row):
        """
        GetRowLabelValue(row) -> String
        
        Returns the specified row label.
        """

    def HideColLabels(self):
        """
        HideColLabels()
        
        Hides the column labels by calling SetColLabelSize() with a size of 0.
        """

    def HideRowLabels(self):
        """
        HideRowLabels()
        
        Hides the row labels by calling SetRowLabelSize() with a size of 0.
        """

    def SetColLabelAlignment(self, horiz, vert):
        """
        SetColLabelAlignment(horiz, vert)
        
        Sets the horizontal and vertical alignment of column label text.
        """

    def SetColLabelTextOrientation(self, textOrientation):
        """
        SetColLabelTextOrientation(textOrientation)
        
        Sets the orientation of the column labels (either wxHORIZONTAL or
        wxVERTICAL).
        """

    def SetColLabelValue(self, col, value):
        """
        SetColLabelValue(col, value)
        
        Set the value for the given column label.
        """

    def SetLabelBackgroundColour(self, colour):
        """
        SetLabelBackgroundColour(colour)
        
        Sets the background colour for row and column labels.
        """

    def SetLabelFont(self, font):
        """
        SetLabelFont(font)
        
        Sets the font for row and column labels.
        """

    def SetLabelTextColour(self, colour):
        """
        SetLabelTextColour(colour)
        
        Sets the colour for row and column label text.
        """

    def SetRowLabelAlignment(self, horiz, vert):
        """
        SetRowLabelAlignment(horiz, vert)
        
        Sets the horizontal and vertical alignment of row label text.
        """

    def SetRowLabelValue(self, row, value):
        """
        SetRowLabelValue(row, value)
        
        Sets the value for the given row label.
        """

    def SetUseNativeColLabels(self, native=True):
        """
        SetUseNativeColLabels(native=True)
        
        Call this in order to make the column labels use a native look by
        using wxRendererNative::DrawHeaderButton() internally.
        """

    def UseNativeColHeader(self, native=True):
        """
        UseNativeColHeader(native=True)
        
        Enable the use of native header window for column labels.
        """

    def GetCellAlignment(self, row, col):
        """
        GetCellAlignment(row, col) -> (horiz, vert)
        
        Sets the arguments to the horizontal and vertical text alignment
        values for the grid cell at the specified location.
        """

    def GetCellBackgroundColour(self, row, col):
        """
        GetCellBackgroundColour(row, col) -> wx.Colour
        
        Returns the background colour of the cell at the specified location.
        """

    def GetCellFont(self, row, col):
        """
        GetCellFont(row, col) -> wx.Font
        
        Returns the font for text in the grid cell at the specified location.
        """

    def GetCellTextColour(self, row, col):
        """
        GetCellTextColour(row, col) -> wx.Colour
        
        Returns the text colour for the grid cell at the specified location.
        """

    def GetDefaultCellAlignment(self):
        """
        GetDefaultCellAlignment() -> (horiz, vert)
        
        Returns the default cell alignment.
        """

    def GetDefaultCellBackgroundColour(self):
        """
        GetDefaultCellBackgroundColour() -> wx.Colour
        
        Returns the current default background colour for grid cells.
        """

    def GetDefaultCellFont(self):
        """
        GetDefaultCellFont() -> wx.Font
        
        Returns the current default font for grid cell text.
        """

    def GetDefaultCellTextColour(self):
        """
        GetDefaultCellTextColour() -> wx.Colour
        
        Returns the current default colour for grid cell text.
        """

    def SetCellAlignment(self, *args, **kw):
        """
        SetCellAlignment(row, col, horiz, vert)
        SetCellAlignment(align, row, col)
        
        Sets the horizontal and vertical alignment for grid cell text at the
        specified location.
        """

    def SetCellBackgroundColour(self, row, col, colour):
        """
        SetCellBackgroundColour(row, col, colour)
        
        Set the background colour for the given cell or all cells by default.
        """

    def SetCellFont(self, row, col, font):
        """
        SetCellFont(row, col, font)
        
        Sets the font for text in the grid cell at the specified location.
        """

    def SetCellTextColour(self, *args, **kw):
        """
        SetCellTextColour(row, col, colour)
        SetCellTextColour(val, row, col)
        SetCellTextColour(colour)
        
        Sets the text colour for the given cell.
        """

    def SetDefaultCellAlignment(self, horiz, vert):
        """
        SetDefaultCellAlignment(horiz, vert)
        
        Sets the default horizontal and vertical alignment for grid cell text.
        """

    def SetDefaultCellBackgroundColour(self, colour):
        """
        SetDefaultCellBackgroundColour(colour)
        
        Sets the default background colour for grid cells.
        """

    def SetDefaultCellFont(self, font):
        """
        SetDefaultCellFont(font)
        
        Sets the default font to be used for grid cell text.
        """

    def SetDefaultCellTextColour(self, colour):
        """
        SetDefaultCellTextColour(colour)
        
        Sets the current default colour for grid cell text.
        """

    def CanEnableCellControl(self):
        """
        CanEnableCellControl() -> bool
        
        Returns true if the in-place edit control for the current grid cell
        can be used and false otherwise.
        """

    def DisableCellEditControl(self):
        """
        DisableCellEditControl()
        
        Disables in-place editing of grid cells.
        """

    def EnableCellEditControl(self, enable=True):
        """
        EnableCellEditControl(enable=True)
        
        Enables or disables in-place editing of grid cell data.
        """

    def EnableEditing(self, edit):
        """
        EnableEditing(edit)
        
        Makes the grid globally editable or read-only.
        """

    def GetCellEditor(self, row, col):
        """
        GetCellEditor(row, col) -> GridCellEditor
        
        Returns a pointer to the editor for the cell at the specified
        location.
        """

    def GetCellRenderer(self, row, col):
        """
        GetCellRenderer(row, col) -> GridCellRenderer
        
        Returns a pointer to the renderer for the grid cell at the specified
        location.
        """

    def GetCellValue(self, *args, **kw):
        """
        GetCellValue(row, col) -> String
        GetCellValue(coords) -> String
        
        Returns the string contained in the cell at the specified location.
        """

    def GetDefaultEditor(self):
        """
        GetDefaultEditor() -> GridCellEditor
        
        Returns a pointer to the current default grid cell editor.
        """

    def GetDefaultEditorForCell(self, *args, **kw):
        """
        GetDefaultEditorForCell(row, col) -> GridCellEditor
        GetDefaultEditorForCell(c) -> GridCellEditor
        
        Returns the default editor for the specified cell.
        """

    def GetDefaultEditorForType(self, typeName):
        """
        GetDefaultEditorForType(typeName) -> GridCellEditor
        
        Returns the default editor for the cells containing values of the
        given type.
        """

    def GetDefaultRenderer(self):
        """
        GetDefaultRenderer() -> GridCellRenderer
        
        Returns a pointer to the current default grid cell renderer.
        """

    def GetDefaultRendererForCell(self, row, col):
        """
        GetDefaultRendererForCell(row, col) -> GridCellRenderer
        
        Returns the default renderer for the given cell.
        """

    def GetDefaultRendererForType(self, typeName):
        """
        GetDefaultRendererForType(typeName) -> GridCellRenderer
        
        Returns the default renderer for the cell containing values of the
        given type.
        """

    def HideCellEditControl(self):
        """
        HideCellEditControl()
        
        Hides the in-place cell edit control.
        """

    def IsCellEditControlEnabled(self):
        """
        IsCellEditControlEnabled() -> bool
        
        Returns true if the in-place edit control is currently enabled.
        """

    def IsCellEditControlShown(self):
        """
        IsCellEditControlShown() -> bool
        
        Returns true if the in-place edit control is currently shown.
        """

    def IsCurrentCellReadOnly(self):
        """
        IsCurrentCellReadOnly() -> bool
        
        Returns true if the current cell is read-only.
        """

    def IsEditable(self):
        """
        IsEditable() -> bool
        
        Returns false if the whole grid has been set as read-only or true
        otherwise.
        """

    def IsReadOnly(self, row, col):
        """
        IsReadOnly(row, col) -> bool
        
        Returns true if the cell at the specified location can't be edited.
        """

    def RegisterDataType(self, typeName, renderer, editor):
        """
        RegisterDataType(typeName, renderer, editor)
        
        Register a new data type.
        """

    def SaveEditControlValue(self):
        """
        SaveEditControlValue()
        
        Sets the value of the current grid cell to the current in-place edit
        control value.
        """

    def SetCellEditor(self, row, col, editor):
        """
        SetCellEditor(row, col, editor)
        
        Sets the editor for the grid cell at the specified location.
        """

    def SetCellRenderer(self, row, col, renderer):
        """
        SetCellRenderer(row, col, renderer)
        
        Sets the renderer for the grid cell at the specified location.
        """

    def SetCellValue(self, *args, **kw):
        """
        SetCellValue(row, col, s)
        SetCellValue(coords, s)
        
        Sets the string value for the cell at the specified location.
        """

    def SetColFormatBool(self, col):
        """
        SetColFormatBool(col)
        
        Sets the specified column to display boolean values.
        """

    def SetColFormatCustom(self, col, typeName):
        """
        SetColFormatCustom(col, typeName)
        
        Sets the specified column to display data in a custom format.
        """

    def SetColFormatFloat(self, col, width=-1, precision=-1):
        """
        SetColFormatFloat(col, width=-1, precision=-1)
        
        Sets the specified column to display floating point values with the
        given width and precision.
        """

    def SetColFormatNumber(self, col):
        """
        SetColFormatNumber(col)
        
        Sets the specified column to display integer values.
        """

    def SetDefaultEditor(self, editor):
        """
        SetDefaultEditor(editor)
        
        Sets the default editor for grid cells.
        """

    def SetDefaultRenderer(self, renderer):
        """
        SetDefaultRenderer(renderer)
        
        Sets the default renderer for grid cells.
        """

    def SetReadOnly(self, row, col, isReadOnly=True):
        """
        SetReadOnly(row, col, isReadOnly=True)
        
        Makes the cell at the specified location read-only or editable.
        """

    def ShowCellEditControl(self):
        """
        ShowCellEditControl()
        
        Displays the in-place cell edit control for the current cell.
        """

    def AutoSize(self):
        """
        AutoSize()
        
        Automatically sets the height and width of all rows and columns to fit
        their contents.
        """

    def AutoSizeColLabelSize(self, col):
        """
        AutoSizeColLabelSize(col)
        
        Automatically adjusts width of the column to fit its label.
        """

    def AutoSizeColumn(self, col, setAsMin=True):
        """
        AutoSizeColumn(col, setAsMin=True)
        
        Automatically sizes the column to fit its contents.
        """

    def AutoSizeColumns(self, setAsMin=True):
        """
        AutoSizeColumns(setAsMin=True)
        
        Automatically sizes all columns to fit their contents.
        """

    def AutoSizeRow(self, row, setAsMin=True):
        """
        AutoSizeRow(row, setAsMin=True)
        
        Automatically sizes the row to fit its contents.
        """

    def AutoSizeRowLabelSize(self, col):
        """
        AutoSizeRowLabelSize(col)
        
        Automatically adjusts height of the row to fit its label.
        """

    def AutoSizeRows(self, setAsMin=True):
        """
        AutoSizeRows(setAsMin=True)
        
        Automatically sizes all rows to fit their contents.
        """

    def GetCellOverflow(self, row, col):
        """
        GetCellOverflow(row, col) -> bool
        
        Returns true if the cell value can overflow.
        """

    def GetColLabelSize(self):
        """
        GetColLabelSize() -> int
        
        Returns the current height of the column labels.
        """

    def GetColMinimalAcceptableWidth(self):
        """
        GetColMinimalAcceptableWidth() -> int
        
        Returns the minimal width to which a column may be resized.
        """

    def GetColSize(self, col):
        """
        GetColSize(col) -> int
        
        Returns the width of the specified column.
        """

    def IsColShown(self, col):
        """
        IsColShown(col) -> bool
        
        Returns true if the specified column is not currently hidden.
        """

    def GetDefaultCellOverflow(self):
        """
        GetDefaultCellOverflow() -> bool
        
        Returns true if the cells can overflow by default.
        """

    def GetDefaultColLabelSize(self):
        """
        GetDefaultColLabelSize() -> int
        
        Returns the default height for column labels.
        """

    def GetDefaultColSize(self):
        """
        GetDefaultColSize() -> int
        
        Returns the current default width for grid columns.
        """

    def GetDefaultRowLabelSize(self):
        """
        GetDefaultRowLabelSize() -> int
        
        Returns the default width for the row labels.
        """

    def GetDefaultRowSize(self):
        """
        GetDefaultRowSize() -> int
        
        Returns the current default height for grid rows.
        """

    def GetRowMinimalAcceptableHeight(self):
        """
        GetRowMinimalAcceptableHeight() -> int
        
        Returns the minimal size to which rows can be resized.
        """

    def GetRowLabelSize(self):
        """
        GetRowLabelSize() -> int
        
        Returns the current width of the row labels.
        """

    def GetRowSize(self, row):
        """
        GetRowSize(row) -> int
        
        Returns the height of the specified row.
        """

    def IsRowShown(self, row):
        """
        IsRowShown(row) -> bool
        
        Returns true if the specified row is not currently hidden.
        """

    def SetCellOverflow(self, row, col, allow):
        """
        SetCellOverflow(row, col, allow)
        
        Sets the overflow permission of the cell.
        """

    def SetColLabelSize(self, height):
        """
        SetColLabelSize(height)
        
        Sets the height of the column labels.
        """

    def SetColMinimalAcceptableWidth(self, width):
        """
        SetColMinimalAcceptableWidth(width)
        
        Sets the minimal width to which the user can resize columns.
        """

    def SetColMinimalWidth(self, col, width):
        """
        SetColMinimalWidth(col, width)
        
        Sets the minimal width for the specified column col.
        """

    def SetColSize(self, col, width):
        """
        SetColSize(col, width)
        
        Sets the width of the specified column.
        """

    def HideCol(self, col):
        """
        HideCol(col)
        
        Hides the specified column.
        """

    def ShowCol(self, col):
        """
        ShowCol(col)
        
        Shows the previously hidden column by resizing it to non-0 size.
        """

    def SetDefaultCellOverflow(self, allow):
        """
        SetDefaultCellOverflow(allow)
        
        Sets the default overflow permission of the cells.
        """

    def SetDefaultColSize(self, width, resizeExistingCols=False):
        """
        SetDefaultColSize(width, resizeExistingCols=False)
        
        Sets the default width for columns in the grid.
        """

    def SetDefaultRowSize(self, height, resizeExistingRows=False):
        """
        SetDefaultRowSize(height, resizeExistingRows=False)
        
        Sets the default height for rows in the grid.
        """

    def SetRowLabelSize(self, width):
        """
        SetRowLabelSize(width)
        
        Sets the width of the row labels.
        """

    def SetRowMinimalAcceptableHeight(self, height):
        """
        SetRowMinimalAcceptableHeight(height)
        
        Sets the minimal row height used by default.
        """

    def SetRowMinimalHeight(self, row, height):
        """
        SetRowMinimalHeight(row, height)
        
        Sets the minimal height for the specified row.
        """

    def SetRowSize(self, row, height):
        """
        SetRowSize(row, height)
        
        Sets the height of the specified row.
        """

    def HideRow(self, col):
        """
        HideRow(col)
        
        Hides the specified row.
        """

    def ShowRow(self, col):
        """
        ShowRow(col)
        
        Shows the previously hidden row.
        """

    def GetColSizes(self):
        """
        GetColSizes() -> GridSizesInfo
        
        Get size information for all columns at once.
        """

    def GetRowSizes(self):
        """
        GetRowSizes() -> GridSizesInfo
        
        Get size information for all row at once.
        """

    def SetColSizes(self, sizeInfo):
        """
        SetColSizes(sizeInfo)
        
        Restore all columns sizes.
        """

    def SetRowSizes(self, sizeInfo):
        """
        SetRowSizes(sizeInfo)
        
        Restore all rows sizes.
        """

    def SetCellSize(self, row, col, num_rows, num_cols):
        """
        SetCellSize(row, col, num_rows, num_cols)
        
        Set the size of the cell.
        """

    def GetCellSize(self, *args, **kw):
        """
        GetCellSize(row, col, num_rows, num_cols) -> CellSpan
        GetCellSize(coords) -> wx.Size
        
        Get the size of the cell in number of cells covered by it.
        """

    def CanDragCell(self):
        """
        CanDragCell() -> bool
        
        Return true if the dragging of cells is enabled or false otherwise.
        """

    def CanDragColMove(self):
        """
        CanDragColMove() -> bool
        
        Returns true if columns can be moved by dragging with the mouse.
        """

    def CanDragColSize(self, col):
        """
        CanDragColSize(col) -> bool
        
        Returns true if the given column can be resized by dragging with the
        mouse.
        """

    def CanDragGridSize(self):
        """
        CanDragGridSize() -> bool
        
        Return true if the dragging of grid lines to resize rows and columns
        is enabled or false otherwise.
        """

    def CanDragRowSize(self, row):
        """
        CanDragRowSize(row) -> bool
        
        Returns true if the given row can be resized by dragging with the
        mouse.
        """

    def DisableColResize(self, col):
        """
        DisableColResize(col)
        
        Disable interactive resizing of the specified column.
        """

    def DisableRowResize(self, row):
        """
        DisableRowResize(row)
        
        Disable interactive resizing of the specified row.
        """

    def DisableDragColMove(self):
        """
        DisableDragColMove()
        
        Disables column moving by dragging with the mouse.
        """

    def DisableDragColSize(self):
        """
        DisableDragColSize()
        
        Disables column sizing by dragging with the mouse.
        """

    def DisableDragGridSize(self):
        """
        DisableDragGridSize()
        
        Disable mouse dragging of grid lines to resize rows and columns.
        """

    def DisableDragRowSize(self):
        """
        DisableDragRowSize()
        
        Disables row sizing by dragging with the mouse.
        """

    def EnableDragCell(self, enable=True):
        """
        EnableDragCell(enable=True)
        
        Enables or disables cell dragging with the mouse.
        """

    def EnableDragColMove(self, enable=True):
        """
        EnableDragColMove(enable=True)
        
        Enables or disables column moving by dragging with the mouse.
        """

    def EnableDragColSize(self, enable=True):
        """
        EnableDragColSize(enable=True)
        
        Enables or disables column sizing by dragging with the mouse.
        """

    def EnableDragGridSize(self, enable=True):
        """
        EnableDragGridSize(enable=True)
        
        Enables or disables row and column resizing by dragging gridlines with
        the mouse.
        """

    def EnableDragRowSize(self, enable=True):
        """
        EnableDragRowSize(enable=True)
        
        Enables or disables row sizing by dragging with the mouse.
        """

    def GetColAt(self, colPos):
        """
        GetColAt(colPos) -> int
        
        Returns the column ID of the specified column position.
        """

    def GetColPos(self, colID):
        """
        GetColPos(colID) -> int
        
        Returns the position of the specified column.
        """

    def SetColPos(self, colID, newPos):
        """
        SetColPos(colID, newPos)
        
        Sets the position of the specified column.
        """

    def SetColumnsOrder(self, order):
        """
        SetColumnsOrder(order)
        
        Sets the positions of all columns at once.
        """

    def ResetColPos(self):
        """
        ResetColPos()
        
        Resets the position of the columns to the default.
        """

    def GetGridCursorCol(self):
        """
        GetGridCursorCol() -> int
        
        Returns the current grid cell column position.
        """

    def GetGridCursorRow(self):
        """
        GetGridCursorRow() -> int
        
        Returns the current grid cell row position.
        """

    def GoToCell(self, *args, **kw):
        """
        GoToCell(row, col)
        GoToCell(coords)
        
        Make the given cell current and ensure it is visible.
        """

    def MoveCursorDown(self, expandSelection):
        """
        MoveCursorDown(expandSelection) -> bool
        
        Moves the grid cursor down by one row.
        """

    def MoveCursorDownBlock(self, expandSelection):
        """
        MoveCursorDownBlock(expandSelection) -> bool
        
        Moves the grid cursor down in the current column such that it skips to
        the beginning or end of a block of non-empty cells.
        """

    def MoveCursorLeft(self, expandSelection):
        """
        MoveCursorLeft(expandSelection) -> bool
        
        Moves the grid cursor left by one column.
        """

    def MoveCursorLeftBlock(self, expandSelection):
        """
        MoveCursorLeftBlock(expandSelection) -> bool
        
        Moves the grid cursor left in the current row such that it skips to
        the beginning or end of a block of non-empty cells.
        """

    def MoveCursorRight(self, expandSelection):
        """
        MoveCursorRight(expandSelection) -> bool
        
        Moves the grid cursor right by one column.
        """

    def MoveCursorRightBlock(self, expandSelection):
        """
        MoveCursorRightBlock(expandSelection) -> bool
        
        Moves the grid cursor right in the current row such that it skips to
        the beginning or end of a block of non-empty cells.
        """

    def MoveCursorUp(self, expandSelection):
        """
        MoveCursorUp(expandSelection) -> bool
        
        Moves the grid cursor up by one row.
        """

    def MoveCursorUpBlock(self, expandSelection):
        """
        MoveCursorUpBlock(expandSelection) -> bool
        
        Moves the grid cursor up in the current column such that it skips to
        the beginning or end of a block of non-empty cells.
        """

    def MovePageDown(self):
        """
        MovePageDown() -> bool
        
        Moves the grid cursor down by some number of rows so that the previous
        bottom visible row becomes the top visible row.
        """

    def MovePageUp(self):
        """
        MovePageUp() -> bool
        
        Moves the grid cursor up by some number of rows so that the previous
        top visible row becomes the bottom visible row.
        """

    def SetGridCursor(self, *args, **kw):
        """
        SetGridCursor(row, col)
        SetGridCursor(coords)
        
        Set the grid cursor to the specified cell.
        """

    def SetTabBehaviour(self, behaviour):
        """
        SetTabBehaviour(behaviour)
        
        Set the grid's behaviour when the user presses the TAB key.
        """

    def ClearSelection(self):
        """
        ClearSelection()
        
        Deselects all cells that are currently selected.
        """

    def DeselectRow(self, row):
        """
        DeselectRow(row)
        
        Deselects a row of cells.
        """

    def DeselectCol(self, col):
        """
        DeselectCol(col)
        
        Deselects a column of cells.
        """

    def DeselectCell(self, row, col):
        """
        DeselectCell(row, col)
        
        Deselects a cell.
        """

    def GetSelectedCells(self):
        """
        GetSelectedCells() -> GridCellCoordsArray
        
        Returns an array of individually selected cells.
        """

    def GetSelectedCols(self):
        """
        GetSelectedCols() -> ArrayInt
        
        Returns an array of selected columns.
        """

    def GetSelectedRows(self):
        """
        GetSelectedRows() -> ArrayInt
        
        Returns an array of selected rows.
        """

    def GetSelectionBackground(self):
        """
        GetSelectionBackground() -> wx.Colour
        
        Returns the colour used for drawing the selection background.
        """

    def GetSelectionBlockBottomRight(self):
        """
        GetSelectionBlockBottomRight() -> GridCellCoordsArray
        
        Returns an array of the bottom right corners of blocks of selected
        cells.
        """

    def GetSelectionBlockTopLeft(self):
        """
        GetSelectionBlockTopLeft() -> GridCellCoordsArray
        
        Returns an array of the top left corners of blocks of selected cells.
        """

    def GetSelectionForeground(self):
        """
        GetSelectionForeground() -> wx.Colour
        
        Returns the colour used for drawing the selection foreground.
        """

    def GetSelectionMode(self):
        """
        GetSelectionMode() -> GridSelectionModes
        
        Returns the current selection mode.
        """

    def IsInSelection(self, *args, **kw):
        """
        IsInSelection(row, col) -> bool
        IsInSelection(coords) -> bool
        
        Returns true if the given cell is selected.
        """

    def IsSelection(self):
        """
        IsSelection() -> bool
        
        Returns true if there are currently any selected cells, rows, columns
        or blocks.
        """

    def SelectAll(self):
        """
        SelectAll()
        
        Selects all cells in the grid.
        """

    def SelectBlock(self, *args, **kw):
        """
        SelectBlock(topRow, leftCol, bottomRow, rightCol, addToSelected=False)
        SelectBlock(topLeft, bottomRight, addToSelected=False)
        
        Selects a rectangular block of cells.
        """

    def SelectCol(self, col, addToSelected=False):
        """
        SelectCol(col, addToSelected=False)
        
        Selects the specified column.
        """

    def SelectRow(self, row, addToSelected=False):
        """
        SelectRow(row, addToSelected=False)
        
        Selects the specified row.
        """

    def SetSelectionBackground(self, c):
        """
        SetSelectionBackground(c)
        
        Set the colour to be used for drawing the selection background.
        """

    def SetSelectionForeground(self, c):
        """
        SetSelectionForeground(c)
        
        Set the colour to be used for drawing the selection foreground.
        """

    def SetSelectionMode(self, selmode):
        """
        SetSelectionMode(selmode)
        
        Set the selection behaviour of the grid.
        """

    def GetScrollLineX(self):
        """
        GetScrollLineX() -> int
        
        Returns the number of pixels per horizontal scroll increment.
        """

    def GetScrollLineY(self):
        """
        GetScrollLineY() -> int
        
        Returns the number of pixels per vertical scroll increment.
        """

    def IsVisible(self, *args, **kw):
        """
        IsVisible(row, col, wholeCellVisible=True) -> bool
        IsVisible(coords, wholeCellVisible=True) -> bool
        
        Returns true if a cell is either entirely or at least partially
        visible in the grid window.
        """

    def MakeCellVisible(self, *args, **kw):
        """
        MakeCellVisible(row, col)
        MakeCellVisible(coords)
        
        Brings the specified cell into the visible grid cell area with minimal
        scrolling.
        """

    def SetScrollLineX(self, x):
        """
        SetScrollLineX(x)
        
        Sets the number of pixels per horizontal scroll increment.
        """

    def SetScrollLineY(self, y):
        """
        SetScrollLineY(y)
        
        Sets the number of pixels per vertical scroll increment.
        """

    def BlockToDeviceRect(self, topLeft, bottomRight):
        """
        BlockToDeviceRect(topLeft, bottomRight) -> wx.Rect
        
        Convert grid cell coordinates to grid window pixel coordinates.
        """

    def CellToRect(self, *args, **kw):
        """
        CellToRect(row, col) -> wx.Rect
        CellToRect(coords) -> wx.Rect
        
        Return the rectangle corresponding to the grid cell's size and
        position in logical coordinates.
        """

    def XToCol(self, x, clipToMinMax=False):
        """
        XToCol(x, clipToMinMax=False) -> int
        
        Returns the column at the given pixel position.
        """

    def XToEdgeOfCol(self, x):
        """
        XToEdgeOfCol(x) -> int
        
        Returns the column whose right hand edge is close to the given logical
        x position.
        """

    def XYToCell(self, *args, **kw):
        """
        XYToCell(x, y) -> GridCellCoords
        XYToCell(pos) -> GridCellCoords
        
        Translates logical pixel coordinates to the grid cell coordinates.
        """

    def YToEdgeOfRow(self, y):
        """
        YToEdgeOfRow(y) -> int
        
        Returns the row whose bottom edge is close to the given logical y
        position.
        """

    def YToRow(self, y, clipToMinMax=False):
        """
        YToRow(y, clipToMinMax=False) -> int
        
        Returns the grid row that corresponds to the logical y coordinate.
        """

    def AppendCols(self, numCols=1, updateLabels=True):
        """
        AppendCols(numCols=1, updateLabels=True) -> bool
        
        Appends one or more new columns to the right of the grid.
        """

    def AppendRows(self, numRows=1, updateLabels=True):
        """
        AppendRows(numRows=1, updateLabels=True) -> bool
        
        Appends one or more new rows to the bottom of the grid.
        """

    def AreHorzGridLinesClipped(self):
        """
        AreHorzGridLinesClipped() -> bool
        
        Return true if the horizontal grid lines stop at the last column
        boundary or false if they continue to the end of the window.
        """

    def AreVertGridLinesClipped(self):
        """
        AreVertGridLinesClipped() -> bool
        
        Return true if the vertical grid lines stop at the last row boundary
        or false if they continue to the end of the window.
        """

    def BeginBatch(self):
        """
        BeginBatch()
        
        Increments the grid's batch count.
        """

    def ClearGrid(self):
        """
        ClearGrid()
        
        Clears all data in the underlying grid table and repaints the grid.
        """

    def ClipHorzGridLines(self, clip):
        """
        ClipHorzGridLines(clip)
        
        Change whether the horizontal grid lines are clipped by the end of the
        last column.
        """

    def ClipVertGridLines(self, clip):
        """
        ClipVertGridLines(clip)
        
        Change whether the vertical grid lines are clipped by the end of the
        last row.
        """

    def DeleteCols(self, pos=0, numCols=1, updateLabels=True):
        """
        DeleteCols(pos=0, numCols=1, updateLabels=True) -> bool
        
        Deletes one or more columns from a grid starting at the specified
        position.
        """

    def DeleteRows(self, pos=0, numRows=1, updateLabels=True):
        """
        DeleteRows(pos=0, numRows=1, updateLabels=True) -> bool
        
        Deletes one or more rows from a grid starting at the specified
        position.
        """

    def EndBatch(self):
        """
        EndBatch()
        
        Decrements the grid's batch count.
        """

    def Fit(self):
        """
        Fit()
        
        Overridden wxWindow method.
        """

    def ForceRefresh(self):
        """
        ForceRefresh()
        
        Causes immediate repainting of the grid.
        """

    def GetBatchCount(self):
        """
        GetBatchCount() -> int
        
        Returns the number of times that BeginBatch() has been called without
        (yet) matching calls to EndBatch().
        """

    def GetNumberCols(self):
        """
        GetNumberCols() -> int
        
        Returns the total number of grid columns.
        """

    def GetNumberRows(self):
        """
        GetNumberRows() -> int
        
        Returns the total number of grid rows.
        """

    def GetOrCreateCellAttr(self, row, col):
        """
        GetOrCreateCellAttr(row, col) -> GridCellAttr
        
        Returns the attribute for the given cell creating one if necessary.
        """

    def GetTable(self):
        """
        GetTable() -> GridTableBase
        
        Returns a base pointer to the current table object.
        """

    def InsertCols(self, pos=0, numCols=1, updateLabels=True):
        """
        InsertCols(pos=0, numCols=1, updateLabels=True) -> bool
        
        Inserts one or more new columns into a grid with the first new column
        at the specified position.
        """

    def InsertRows(self, pos=0, numRows=1, updateLabels=True):
        """
        InsertRows(pos=0, numRows=1, updateLabels=True) -> bool
        
        Inserts one or more new rows into a grid with the first new row at the
        specified position.
        """

    def RefreshAttr(self, row, col):
        """
        RefreshAttr(row, col)
        
        Invalidates the cached attribute for the given cell.
        """

    def Render(self, dc, pos=wx.DefaultPosition, size=wx.DefaultSize, topLeft=GridCellCoords(-1,-1), bottomRight=GridCellCoords(-1,-1), style=GRID_DRAW_DEFAULT):
        """
        Render(dc, pos=wx.DefaultPosition, size=wx.DefaultSize, topLeft=GridCellCoords(-1,-1), bottomRight=GridCellCoords(-1,-1), style=GRID_DRAW_DEFAULT)
        
        Draws part or all of a wxGrid on a wxDC for printing or display.
        """

    def SetAttr(self, row, col, attr):
        """
        SetAttr(row, col, attr)
        
        Sets the cell attributes for the specified cell.
        """

    def SetColAttr(self, col, attr):
        """
        SetColAttr(col, attr)
        
        Sets the cell attributes for all cells in the specified column.
        """

    def SetMargins(self, extraWidth, extraHeight):
        """
        SetMargins(extraWidth, extraHeight)
        
        Sets the extra margins used around the grid area.
        """

    def SetRowAttr(self, row, attr):
        """
        SetRowAttr(row, attr)
        
        Sets the cell attributes for all cells in the specified row.
        """

    def CalcRowLabelsExposed(self, reg):
        """
        CalcRowLabelsExposed(reg) -> ArrayInt
        
        Appends one or more new columns to the right of the grid.
        """

    def CalcColLabelsExposed(self, reg):
        """
        CalcColLabelsExposed(reg) -> ArrayInt
        
        Appends one or more new columns to the right of the grid.
        """

    def CalcCellsExposed(self, reg):
        """
        CalcCellsExposed(reg) -> GridCellCoordsArray
        
        Appends one or more new columns to the right of the grid.
        """

    def GetSortingColumn(self):
        """
        GetSortingColumn() -> int
        
        Return the column in which the sorting indicator is currently
        displayed.
        """

    def IsSortingBy(self, col):
        """
        IsSortingBy(col) -> bool
        
        Return true if this column is currently used for sorting.
        """

    def IsSortOrderAscending(self):
        """
        IsSortOrderAscending() -> bool
        
        Return true if the current sorting order is ascending or false if it
        is descending.
        """

    def SetSortingColumn(self, col, ascending=True):
        """
        SetSortingColumn(col, ascending=True)
        
        Set the column to display the sorting indicator in and its direction.
        """

    def UnsetSortingColumn(self):
        """
        UnsetSortingColumn()
        
        Remove any currently shown sorting indicator.
        """

    def GetGridWindow(self):
        """
        GetGridWindow() -> wx.Window
        
        Return the main grid window containing the grid cells.
        """

    def GetGridRowLabelWindow(self):
        """
        GetGridRowLabelWindow() -> wx.Window
        
        Return the row labels window.
        """

    def GetGridColLabelWindow(self):
        """
        GetGridColLabelWindow() -> wx.Window
        
        Return the column labels window.
        """

    def GetGridCornerLabelWindow(self):
        """
        GetGridCornerLabelWindow() -> wx.Window
        
        Return the window in the top left grid corner.
        """

    def GetGridColHeader(self):
        """
        GetGridColHeader() -> wx.HeaderCtrl
        
        Return the header control used for column labels display.
        """

    def DrawCellHighlight(self, dc, attr):
        """
        DrawCellHighlight(dc, attr)
        """

    def DrawRowLabels(self, dc, rows):
        """
        DrawRowLabels(dc, rows)
        """

    def DrawRowLabel(self, dc, row):
        """
        DrawRowLabel(dc, row)
        """

    def DrawColLabels(self, dc, cols):
        """
        DrawColLabels(dc, cols)
        """

    def DrawColLabel(self, dc, col):
        """
        DrawColLabel(dc, col)
        """

    def DrawCornerLabel(self, dc):
        """
        DrawCornerLabel(dc)
        """

    def DrawTextRectangle(self, *args, **kw):
        """
        DrawTextRectangle(dc, text, rect, horizontalAlignment=wx.ALIGN_LEFT, verticalAlignment=wx.ALIGN_TOP, textOrientation=wx.HORIZONTAL)
        DrawTextRectangle(dc, lines, rect, horizontalAlignment=wx.ALIGN_LEFT, verticalAlignment=wx.ALIGN_TOP, textOrientation=wx.HORIZONTAL)
        """

    def GetCellHighlightColour(self):
        """
        GetCellHighlightColour() -> wx.Colour
        """

    def GetCellHighlightPenWidth(self):
        """
        GetCellHighlightPenWidth() -> int
        """

    def GetCellHighlightROPenWidth(self):
        """
        GetCellHighlightROPenWidth() -> int
        """

    def SetCellHighlightColour(self):
        """
        SetCellHighlightColour()
        """

    def SetCellHighlightPenWidth(self, width):
        """
        SetCellHighlightPenWidth(width)
        """

    def SetCellHighlightROPenWidth(self, width):
        """
        SetCellHighlightROPenWidth(width)
        """

    def SetTable(self, table, takeOwnership=False, selmode=SelectCells):
        """
        Set the Grid Table to be used by this grid.
        """

    wxGridSelectCells = SelectCells
    wxGridSelectRows = SelectRows
    wxGridSelectColumns = SelectColumns
    wxGridSelectRowsOrColumns = SelectRowsOrColumns
    BatchCount = property(None, None)
    CellHighlightColour = property(None, None)
    CellHighlightPenWidth = property(None, None)
    CellHighlightROPenWidth = property(None, None)
    ColLabelSize = property(None, None)
    ColLabelTextOrientation = property(None, None)
    ColMinimalAcceptableWidth = property(None, None)
    ColSizes = property(None, None)
    DefaultCellBackgroundColour = property(None, None)
    DefaultCellFont = property(None, None)
    DefaultCellOverflow = property(None, None)
    DefaultCellTextColour = property(None, None)
    DefaultColLabelSize = property(None, None)
    DefaultColSize = property(None, None)
    DefaultEditor = property(None, None)
    DefaultGridLinePen = property(None, None)
    DefaultRenderer = property(None, None)
    DefaultRowLabelSize = property(None, None)
    DefaultRowSize = property(None, None)
    GridColHeader = property(None, None)
    GridColLabelWindow = property(None, None)
    GridCornerLabelWindow = property(None, None)
    GridCursorCol = property(None, None)
    GridCursorRow = property(None, None)
    GridLineColour = property(None, None)
    GridRowLabelWindow = property(None, None)
    GridWindow = property(None, None)
    LabelBackgroundColour = property(None, None)
    LabelFont = property(None, None)
    LabelTextColour = property(None, None)
    NumberCols = property(None, None)
    NumberRows = property(None, None)
    RowLabelSize = property(None, None)
    RowMinimalAcceptableHeight = property(None, None)
    RowSizes = property(None, None)
    ScrollLineX = property(None, None)
    ScrollLineY = property(None, None)
    SelectedCells = property(None, None)
    SelectedCols = property(None, None)
    SelectedRows = property(None, None)
    SelectionBackground = property(None, None)
    SelectionBlockBottomRight = property(None, None)
    SelectionBlockTopLeft = property(None, None)
    SelectionForeground = property(None, None)
    SelectionMode = property(None, None)
    SortingColumn = property(None, None)
    Table = property(None, None)

    def CanHaveAttributes(self):
        """
        CanHaveAttributes() -> bool
        
        Returns true if this grid has support for cell attributes.
        """

    def GetColMinimalWidth(self, col):
        """
        GetColMinimalWidth(col) -> int
        
        Get the minimal width of the given column/row.
        """

    def GetColRight(self, col):
        """
        GetColRight(col) -> int
        
        Returns the coordinate of the right border specified column.
        """

    def GetColLeft(self, col):
        """
        GetColLeft(col) -> int
        
        Returns the coordinate of the left border specified column.
        """

    def GetRowMinimalHeight(self, col):
        """
        GetRowMinimalHeight(col) -> int
        
        Returns the minimal size for the given column.
        """
# end of class Grid


class GridUpdateLocker(object):
    """
    GridUpdateLocker(grid=None)
    
    This small class can be used to prevent wxGrid from redrawing during
    its lifetime by calling wxGrid::BeginBatch() in its constructor and
    wxGrid::EndBatch() in its destructor.
    """

    def __init__(self, grid=None):
        """
        GridUpdateLocker(grid=None)
        
        This small class can be used to prevent wxGrid from redrawing during
        its lifetime by calling wxGrid::BeginBatch() in its constructor and
        wxGrid::EndBatch() in its destructor.
        """

    def Create(self, grid):
        """
        Create(grid)
        
        This method can be called if the object had been constructed using the
        default constructor.
        """

    def __enter__(self):
        """
        
        """

    def __exit__(self, exc_type, exc_val, exc_tb):
        """
        
        """
# end of class GridUpdateLocker


class GridEvent(wx.NotifyEvent):
    """
    GridEvent()
    GridEvent(id, type, obj, row=-1, col=-1, x=-1, y=-1, sel=True, kbd=wx.KeyboardState())
    
    This event class contains information about various grid events.
    """

    def __init__(self, *args, **kw):
        """
        GridEvent()
        GridEvent(id, type, obj, row=-1, col=-1, x=-1, y=-1, sel=True, kbd=wx.KeyboardState())
        
        This event class contains information about various grid events.
        """

    def AltDown(self):
        """
        AltDown() -> bool
        
        Returns true if the Alt key was down at the time of the event.
        """

    def ControlDown(self):
        """
        ControlDown() -> bool
        
        Returns true if the Control key was down at the time of the event.
        """

    def GetCol(self):
        """
        GetCol() -> int
        
        Column at which the event occurred.
        """

    def GetPosition(self):
        """
        GetPosition() -> wx.Point
        
        Position in pixels at which the event occurred.
        """

    def GetRow(self):
        """
        GetRow() -> int
        
        Row at which the event occurred.
        """

    def MetaDown(self):
        """
        MetaDown() -> bool
        
        Returns true if the Meta key was down at the time of the event.
        """

    def Selecting(self):
        """
        Selecting() -> bool
        
        Returns true if the user is selecting grid cells, or false if
        deselecting.
        """

    def ShiftDown(self):
        """
        ShiftDown() -> bool
        
        Returns true if the Shift key was down at the time of the event.
        """
    Col = property(None, None)
    Position = property(None, None)
    Row = property(None, None)
# end of class GridEvent


class GridSizeEvent(wx.NotifyEvent):
    """
    GridSizeEvent()
    GridSizeEvent(id, type, obj, rowOrCol=-1, x=-1, y=-1, kbd=wx.KeyboardState())
    
    This event class contains information about a row/column resize event.
    """

    def __init__(self, *args, **kw):
        """
        GridSizeEvent()
        GridSizeEvent(id, type, obj, rowOrCol=-1, x=-1, y=-1, kbd=wx.KeyboardState())
        
        This event class contains information about a row/column resize event.
        """

    def AltDown(self):
        """
        AltDown() -> bool
        
        Returns true if the Alt key was down at the time of the event.
        """

    def ControlDown(self):
        """
        ControlDown() -> bool
        
        Returns true if the Control key was down at the time of the event.
        """

    def GetPosition(self):
        """
        GetPosition() -> wx.Point
        
        Position in pixels at which the event occurred.
        """

    def GetRowOrCol(self):
        """
        GetRowOrCol() -> int
        
        Row or column at that was resized.
        """

    def MetaDown(self):
        """
        MetaDown() -> bool
        
        Returns true if the Meta key was down at the time of the event.
        """

    def ShiftDown(self):
        """
        ShiftDown() -> bool
        
        Returns true if the Shift key was down at the time of the event.
        """
    Position = property(None, None)
    RowOrCol = property(None, None)
# end of class GridSizeEvent


class GridRangeSelectEvent(wx.NotifyEvent):
    """
    GridRangeSelectEvent()
    GridRangeSelectEvent(id, type, obj, topLeft, bottomRight, sel=True, kbd=wx.KeyboardState())
    """

    def __init__(self, *args, **kw):
        """
        GridRangeSelectEvent()
        GridRangeSelectEvent(id, type, obj, topLeft, bottomRight, sel=True, kbd=wx.KeyboardState())
        """

    def AltDown(self):
        """
        AltDown() -> bool
        
        Returns true if the Alt key was down at the time of the event.
        """

    def ControlDown(self):
        """
        ControlDown() -> bool
        
        Returns true if the Control key was down at the time of the event.
        """

    def GetBottomRightCoords(self):
        """
        GetBottomRightCoords() -> GridCellCoords
        
        Top left corner of the rectangular area that was (de)selected.
        """

    def GetBottomRow(self):
        """
        GetBottomRow() -> int
        
        Bottom row of the rectangular area that was (de)selected.
        """

    def GetLeftCol(self):
        """
        GetLeftCol() -> int
        
        Left column of the rectangular area that was (de)selected.
        """

    def GetRightCol(self):
        """
        GetRightCol() -> int
        
        Right column of the rectangular area that was (de)selected.
        """

    def GetTopLeftCoords(self):
        """
        GetTopLeftCoords() -> GridCellCoords
        
        Top left corner of the rectangular area that was (de)selected.
        """

    def GetTopRow(self):
        """
        GetTopRow() -> int
        
        Top row of the rectangular area that was (de)selected.
        """

    def MetaDown(self):
        """
        MetaDown() -> bool
        
        Returns true if the Meta key was down at the time of the event.
        """

    def Selecting(self):
        """
        Selecting() -> bool
        
        Returns true if the area was selected, false otherwise.
        """

    def ShiftDown(self):
        """
        ShiftDown() -> bool
        
        Returns true if the Shift key was down at the time of the event.
        """
    BottomRightCoords = property(None, None)
    BottomRow = property(None, None)
    LeftCol = property(None, None)
    RightCol = property(None, None)
    TopLeftCoords = property(None, None)
    TopRow = property(None, None)
# end of class GridRangeSelectEvent


class GridEditorCreatedEvent(wx.CommandEvent):
    """
    GridEditorCreatedEvent()
    GridEditorCreatedEvent(id, type, obj, row, col, ctrl)
    """

    def __init__(self, *args, **kw):
        """
        GridEditorCreatedEvent()
        GridEditorCreatedEvent(id, type, obj, row, col, ctrl)
        """

    def GetCol(self):
        """
        GetCol() -> int
        
        Returns the column at which the event occurred.
        """

    def GetControl(self):
        """
        GetControl() -> wx.Control
        
        Returns the edit control.
        """

    def GetRow(self):
        """
        GetRow() -> int
        
        Returns the row at which the event occurred.
        """

    def SetCol(self, col):
        """
        SetCol(col)
        
        Sets the column at which the event occurred.
        """

    def SetControl(self, ctrl):
        """
        SetControl(ctrl)
        
        Sets the edit control.
        """

    def SetRow(self, row):
        """
        SetRow(row)
        
        Sets the row at which the event occurred.
        """
    Col = property(None, None)
    Control = property(None, None)
    Row = property(None, None)
# end of class GridEditorCreatedEvent


GRID_VALUE_STRING =    "string"
GRID_VALUE_BOOL =      "bool"
GRID_VALUE_NUMBER =    "long"
GRID_VALUE_FLOAT =     "double"
GRID_VALUE_CHOICE =    "choice"
GRID_VALUE_TEXT =      "string"
GRID_VALUE_LONG =      "long"
GRID_VALUE_CHOICEINT = "choiceint"
GRID_VALUE_DATETIME =  "datetime"

from collections import namedtuple
_im_GridCellCoords = namedtuple('_im_GridCellCoords', ['Row', 'Col'])
del namedtuple

PyGridCellRenderer = wx.deprecated(GridCellRenderer, 'Use GridCellRenderer instead.')

PyGridCellEditor = wx.deprecated(GridCellEditor, 'Use GridCellEditor instead.')

PyGridCellAttrProvider = wx.deprecated(GridCellAttrProvider, 'Use GridCellAttrProvider instead.')

PyGridTableBase = wx.deprecated(GridTableBase, 'Use GridTableBase instead.')

EVT_GRID_CELL_LEFT_CLICK = wx.PyEventBinder( wxEVT_GRID_CELL_LEFT_CLICK )
EVT_GRID_CELL_RIGHT_CLICK = wx.PyEventBinder( wxEVT_GRID_CELL_RIGHT_CLICK )
EVT_GRID_CELL_LEFT_DCLICK = wx.PyEventBinder( wxEVT_GRID_CELL_LEFT_DCLICK )
EVT_GRID_CELL_RIGHT_DCLICK = wx.PyEventBinder( wxEVT_GRID_CELL_RIGHT_DCLICK )
EVT_GRID_LABEL_LEFT_CLICK = wx.PyEventBinder( wxEVT_GRID_LABEL_LEFT_CLICK )
EVT_GRID_LABEL_RIGHT_CLICK = wx.PyEventBinder( wxEVT_GRID_LABEL_RIGHT_CLICK )
EVT_GRID_LABEL_LEFT_DCLICK = wx.PyEventBinder( wxEVT_GRID_LABEL_LEFT_DCLICK )
EVT_GRID_LABEL_RIGHT_DCLICK = wx.PyEventBinder( wxEVT_GRID_LABEL_RIGHT_DCLICK )
EVT_GRID_ROW_SIZE = wx.PyEventBinder( wxEVT_GRID_ROW_SIZE )
EVT_GRID_COL_SIZE = wx.PyEventBinder( wxEVT_GRID_COL_SIZE )
EVT_GRID_RANGE_SELECT = wx.PyEventBinder( wxEVT_GRID_RANGE_SELECT )
EVT_GRID_CELL_CHANGING = wx.PyEventBinder( wxEVT_GRID_CELL_CHANGING )
EVT_GRID_CELL_CHANGED = wx.PyEventBinder( wxEVT_GRID_CELL_CHANGED )
EVT_GRID_SELECT_CELL = wx.PyEventBinder( wxEVT_GRID_SELECT_CELL )
EVT_GRID_EDITOR_SHOWN = wx.PyEventBinder( wxEVT_GRID_EDITOR_SHOWN )
EVT_GRID_EDITOR_HIDDEN = wx.PyEventBinder( wxEVT_GRID_EDITOR_HIDDEN )
EVT_GRID_EDITOR_CREATED = wx.PyEventBinder( wxEVT_GRID_EDITOR_CREATED )
EVT_GRID_CELL_BEGIN_DRAG = wx.PyEventBinder( wxEVT_GRID_CELL_BEGIN_DRAG )
EVT_GRID_COL_MOVE = wx.PyEventBinder( wxEVT_GRID_COL_MOVE )
EVT_GRID_COL_SORT = wx.PyEventBinder( wxEVT_GRID_COL_SORT )
EVT_GRID_TABBING = wx.PyEventBinder( wxEVT_GRID_TABBING )

# The same as above but with the ability to specify an identifier
EVT_GRID_CMD_CELL_LEFT_CLICK =     wx.PyEventBinder( wxEVT_GRID_CELL_LEFT_CLICK,    1 )
EVT_GRID_CMD_CELL_RIGHT_CLICK =    wx.PyEventBinder( wxEVT_GRID_CELL_RIGHT_CLICK,   1 )
EVT_GRID_CMD_CELL_LEFT_DCLICK =    wx.PyEventBinder( wxEVT_GRID_CELL_LEFT_DCLICK,   1 )
EVT_GRID_CMD_CELL_RIGHT_DCLICK =   wx.PyEventBinder( wxEVT_GRID_CELL_RIGHT_DCLICK,  1 )
EVT_GRID_CMD_LABEL_LEFT_CLICK =    wx.PyEventBinder( wxEVT_GRID_LABEL_LEFT_CLICK,   1 )
EVT_GRID_CMD_LABEL_RIGHT_CLICK =   wx.PyEventBinder( wxEVT_GRID_LABEL_RIGHT_CLICK,  1 )
EVT_GRID_CMD_LABEL_LEFT_DCLICK =   wx.PyEventBinder( wxEVT_GRID_LABEL_LEFT_DCLICK,  1 )
EVT_GRID_CMD_LABEL_RIGHT_DCLICK =  wx.PyEventBinder( wxEVT_GRID_LABEL_RIGHT_DCLICK, 1 )
EVT_GRID_CMD_ROW_SIZE =            wx.PyEventBinder( wxEVT_GRID_ROW_SIZE,           1 )
EVT_GRID_CMD_COL_SIZE =            wx.PyEventBinder( wxEVT_GRID_COL_SIZE,           1 )
EVT_GRID_CMD_RANGE_SELECT =        wx.PyEventBinder( wxEVT_GRID_RANGE_SELECT,       1 )
EVT_GRID_CMD_CELL_CHANGING =       wx.PyEventBinder( wxEVT_GRID_CELL_CHANGING,      1 )
EVT_GRID_CMD_CELL_CHANGED =        wx.PyEventBinder( wxEVT_GRID_CELL_CHANGED,       1 )
EVT_GRID_CMD_SELECT_CELL =         wx.PyEventBinder( wxEVT_GRID_SELECT_CELL,        1 )
EVT_GRID_CMD_EDITOR_SHOWN =        wx.PyEventBinder( wxEVT_GRID_EDITOR_SHOWN,       1 )
EVT_GRID_CMD_EDITOR_HIDDEN =       wx.PyEventBinder( wxEVT_GRID_EDITOR_HIDDEN,      1 )
EVT_GRID_CMD_EDITOR_CREATED =      wx.PyEventBinder( wxEVT_GRID_EDITOR_CREATED,     1 )
EVT_GRID_CMD_CELL_BEGIN_DRAG =     wx.PyEventBinder( wxEVT_GRID_CELL_BEGIN_DRAG,    1 )
EVT_GRID_CMD_COL_MOVE =            wx.PyEventBinder( wxEVT_GRID_COL_MOVE,           1 )
EVT_GRID_CMD_COL_SORT =            wx.PyEventBinder( wxEVT_GRID_COL_SORT,           1 )
EVT_GRID_CMD_TABBING =             wx.PyEventBinder( wxEVT_GRID_TABBING,            1 )
#-- end-grid --#



#----------------------------------------------------------------------
# Name:        wx.lib.stattext
# Purpose:     A generic wxGenStaticText class.  Using this should
#              eliminate some of the platform differences in wxStaticText,
#              such as background colours and mouse sensitivity.
#
# Author:      Robin Dunn
#
# Created:     8-July-2002
# Copyright:   (c) 2002-2018 by Total Control Software
# Licence:     wxWindows license
# Tags:        phoenix-port, unittest, documented, py3-port
#----------------------------------------------------------------------
# 12/12/2003 - Jeff Grimmett (grimmtooth@softhome.net)
#
# o 2.5 compatibility update.
# o Untested.
#

"""
:class:`GenStaticText` is a generic implementation of :class:`wx.StaticText`.


Description
===========

:class:`GenStaticText` is a generic implementation of :class:`wx.StaticText`.

Some of the platforms supported by wxPython (most notably GTK), do not
consider :class:`wx.StaticText` as a separate widget; instead, the label is
just drawn directly on its parent window. This essentially bars the use of
almost all mouse events (such as detection of mouse motions, mouse clicks and
so on) on that widget.

Moreover, these platforms do not allow the developer to change the widget's
background colour.

Using :class:`GenStaticText` will overcome all the problems described above,
as it is a generic widget and a real window on its own.


Usage
=====

Sample usage::

    import wx
    import wx.lib.stattext as ST

    app = wx.App(0)

    frame = wx.Frame(None, -1, "wx.lib.stattext Test")
    panel = wx.Panel(frame)

    st1 = ST.GenStaticText(panel, -1, "This is an example of static text", (20, 10))

    st2 = ST.GenStaticText(panel, -1, "Is this yellow?", (20, 70), (120, -1))
    st2.SetBackgroundColour('Yellow')

    ST.GenStaticText(panel, -1, "align center", (160, 70), (120, -1), wx.ALIGN_CENTER)
    ST.GenStaticText(panel, -1, "align right", (300, 70), (120, -1), wx.ALIGN_RIGHT)

    frame.Show()
    app.MainLoop()


"""

import wx

BUFFERED = 0   # In unbuffered mode we can let the theme shine through,
               # otherwise we draw the background ourselves.

if wx.Platform == "__WXMAC__":
    try:
        from Carbon.Appearance import kThemeBrushDialogBackgroundActive
    except ImportError:
        kThemeBrushDialogBackgroundActive = 1

#----------------------------------------------------------------------

class GenStaticText(wx.Control):
    """
    :class:`GenStaticText` is a generic implementation of :class:`wx.StaticText`.
    """
    labelDelta = 1

    def __init__(self, parent, ID=-1, label="",
                 pos=wx.DefaultPosition, size=wx.DefaultSize,
                 style=0,
                 name="genstattext"):
        """
        Default class constructor.

        :param `parent`: parent window, must not be ``None``;
        :param integer `ID`: window identifier. A value of -1 indicates a default value;
        :param string `label`: the static text label (i.e., its text label);
        :param `pos`: the control position. A value of (-1, -1) indicates a default position,
         chosen by either the windowing system or wxPython, depending on platform;
        :param `size`: the control size. A value of (-1, -1) indicates a default size,
         chosen by either the windowing system or wxPython, depending on platform;
        :param integer `style`: the underlying :class:`wx.Control` style;
        :param string `name`: the widget name.

        :type parent: :class:`wx.Window`
        :type pos: tuple or :class:`wx.Point`
        :type size: tuple or :class:`wx.Size`
        """

        wx.Control.__init__(self, parent, ID, pos, size, style|wx.NO_BORDER,
                             wx.DefaultValidator, name)

        wx.Control.SetLabel(self, label) # don't check wx.ST_NO_AUTORESIZE yet
        self.InheritAttributes()
        self.SetInitialSize(size)

        self.Bind(wx.EVT_PAINT, self.OnPaint)
        if BUFFERED:
            self.defBackClr = self.GetBackgroundColour()
            self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)
        else:
            self.SetBackgroundStyle(wx.BG_STYLE_SYSTEM)



    def SetLabel(self, label):
        """
        Sets the static text label and updates the control's size to exactly
        fit the label unless the control has ``wx.ST_NO_AUTORESIZE`` flag.

        :param string `label`: the static text label (i.e., its text label).
        """

        wx.Control.SetLabel(self, label)
        style = self.GetWindowStyleFlag()
        self.InvalidateBestSize()
        if not style & wx.ST_NO_AUTORESIZE:
            self.SetSize(self.GetBestSize())
        self.Refresh()


    def SetFont(self, font):
        """
        Sets the static text font and updates the control's size to exactly
        fit the label unless the control has ``wx.ST_NO_AUTORESIZE`` flag.

        :param wx.Font `font`: a valid font instance, which will be the new font used
         to display the text.
        """

        wx.Control.SetFont(self, font)
        style = self.GetWindowStyleFlag()
        self.InvalidateBestSize()
        if not style & wx.ST_NO_AUTORESIZE:
            self.SetSize(self.GetBestSize())
        self.Refresh()


    def DoGetBestSize(self):
        """
        Overridden base class virtual.  Determines the best size of
        the control based on the label size and the current font.

        .. note:: Overridden from :class:`wx.Control`.
        """

        label = self.GetLabel()
        font = self.GetFont()
        if not font:
            font = wx.SystemSettings.GetFont(wx.SYS_DEFAULT_GUI_FONT)
        dc = wx.ClientDC(self)
        dc.SetFont(font)

        maxWidth = totalHeight = 0
        for line in label.split('\n'):
            if line == '':
                w, h = dc.GetTextExtent('W')  # empty lines have height too
            else:
                w, h = dc.GetTextExtent(line)
            totalHeight += h
            maxWidth = max(maxWidth, w)
        best = wx.Size(maxWidth, totalHeight)
        self.CacheBestSize(best)
        return best


    def Enable(self, enable=True):
        """
        Enable or disable the widget for user input.

        :param bool `enable`: If ``True``, enables the window for input. If
          ``False``, disables the window.

        :returns: ``True`` if the window has been enabled or disabled,
          ``False`` if nothing was done, i.e. if the window had already been
          in the specified state.

        .. note:: Note that when a parent window is disabled, all of its
           children are disabled as well and they are reenabled again when
           the parent is.

        .. note:: Overridden from :class:`wx.Control`.
        """

        retVal = wx.Control.Enable(self, enable)
        self.Refresh()

        return retVal


    def Disable(self):
        """
        Disables the control.

        :returns: ``True`` if the window has been disabled, ``False`` if it had been
         already disabled before the call to this function.

        .. note:: This is functionally equivalent of calling :meth:`~wx.Control.Enable`
           with a ``False`` flag.

        .. note:: Overridden from :class:`wx.Control`.
        """

        retVal = wx.Control.Disable(self)
        self.Refresh()

        return retVal


    def AcceptsFocus(self):
        """
        Can this window be given focus by mouse click?

        .. note:: Overridden from :class:`wx.Control`.
        """

        return False


    def GetDefaultAttributes(self):
        """
        Overridden base class virtual.  By default we should use
        the same font/colour attributes as the native :class:`wx.StaticText`.

        .. note:: Overridden from :class:`wx.Control`.
        """

        return wx.StaticText.GetClassDefaultAttributes()


    def ShouldInheritColours(self):
        """
        Overridden base class virtual.  If the parent has non-default
        colours then we want this control to inherit them.

        .. note:: Overridden from :class:`wx.Control`.
        """

        return True


    def OnPaint(self, event):
        """
        Handles the ``wx.EVT_PAINT`` for :class:`GenStaticText`.

        :param `event`: a :class:`wx.PaintEvent` event to be processed.
        """

        if BUFFERED:
            dc = wx.BufferedPaintDC(self)
        else:
            dc = wx.PaintDC(self)
        width, height = self.GetClientSize()
        if not width or not height:
            return

        if BUFFERED:
            clr = self.GetBackgroundColour()
            if wx.Platform == "__WXMAC__" and clr == self.defBackClr:
                # if colour is still the default then use the theme's  background on Mac
                themeColour = wx.MacThemeColour(kThemeBrushDialogBackgroundActive)
                backBrush = wx.Brush(themeColour)
            else:
                backBrush = wx.Brush(clr, wx.BRUSHSTYLE_SOLID)
            dc.SetBackground(backBrush)
            dc.Clear()

        if self.IsEnabled():
            dc.SetTextForeground(self.GetForegroundColour())
        else:
            dc.SetTextForeground(wx.SystemSettings.GetColour(wx.SYS_COLOUR_GRAYTEXT))

        dc.SetFont(self.GetFont())
        label = self.GetLabel()
        style = self.GetWindowStyleFlag()
        x = y = 0
        for line in label.split('\n'):
            if line == '':
                w, h = self.GetTextExtent('W')  # empty lines have height too
            else:
                w, h = self.GetTextExtent(line)
            if style & wx.ALIGN_RIGHT:
                x = width - w
            if style & wx.ALIGN_CENTER:
                x = (width - w)/2
            dc.DrawText(line, x, y)
            y += h


    def OnEraseBackground(self, event):
        """
        Handles the ``wx.EVT_ERASE_BACKGROUND`` event for :class:`GenStaticText`.

        :param `event`: a :class:`wx.EraseEvent` event to be processed.

        .. note:: This is intentionally empty to reduce flicker.
        """
        pass


#----------------------------------------------------------------------




"""path.py is a utility containing some very simple path utilities for Py to use"""

__author__ = "David N. Mashburn <david.n.mashburn@gmail.com>"
# 07/01/2009

import os
import glob
import subprocess

def pwd():
    print(os.getcwd())

def cd(path,usePrint=True):
    os.chdir(os.path.expandvars(os.path.expanduser(path)))
    if usePrint:
        pwd()

def ls(str='*',fullpath=False):
    g=glob.glob(os.path.expandvars(os.path.expanduser(str)))
    if fullpath:
        for i in g:
            print(i)
    else:
        for i in g:
            print(os.path.split(i)[1])

# This prints the results of running a command in the GUI shell but be warned!
# This is a blocking call, and if you open any kind of interactive
# command-line program like python or bash, the shell will permanantly
# freeze!
# If you want this kind of behavior to be available, please use ipython
# This is NOT a feature or goal of the Py project!
def sx(str=''):
    print(subprocess.getoutput(str))

#cd('~',usePrint=False)



"""
PyColourChooser
Copyright (C) 2002 Michael Gilfix

This file is part of PyColourChooser.

You should have received a file COPYING containing license terms
along with this program; if not, write to Michael Gilfix
(mgilfix@eecs.tufts.edu) for a copy.

This version of PyColourChooser is open source; you can redistribute it and/or
modify it under the terms listed in the file COPYING.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
"""

from __future__ import absolute_import

# 12/14/2003 - Jeff Grimmett (grimmtooth@softhome.net)
#
# o 2.5 compatibility update.
#
# 12/21/2003 - Jeff Grimmett (grimmtooth@softhome.net)
#
# o wxPyColorChooser -> PyColorChooser
# o wxPyColourChooser -> PyColourChooser
#

import  wx

from . import  canvas
import  colorsys

class PyColourSlider(canvas.Canvas):
    """A Pure-Python Colour Slider

    The colour slider displays transitions from value 0 to value 1 in
    HSV, allowing the user to select a colour within the transition
    spectrum.

    This class is best accompanying by a wxSlider that allows the user
    to select a particular colour shade.
    """

    HEIGHT = 172
    WIDTH = 12

    def __init__(self, parent, id, colour=None):
        """Creates a blank slider instance. A colour must be set before the
        slider will be filled in."""
        # Set the base colour first since our base class calls the buffer
        # drawing function
        self.SetBaseColour(colour)

        canvas.Canvas.__init__(self, parent, id, forceClientSize=(self.WIDTH, self.HEIGHT))

    def SetBaseColour(self, colour):
        """Sets the base, or target colour, to use as the central colour
        when calculating colour transitions."""
        self.base_colour = colour

    def GetBaseColour(self):
        """Return the current colour used as a colour base for filling out
        the slider."""
        return self.base_colour

    def GetVFromClick(self, pos):
        """
        Returns the HSV value "V" based on the location of a mouse click at y offset "pos"
        """
        _, height = self.GetClientSize()
        if pos < 0:
            return 1             # Snap to max
        if pos >= height - 1:
            return 0             # Snap to 0

        return 1 - (pos / self.HEIGHT)

    def DrawBuffer(self):
        """Actual implementation of the widget's drawing. We simply draw
        from value 0.0 to value 1.0 in HSV."""
        if self.base_colour is None:
            return

        target_red = self.base_colour.Red()
        target_green = self.base_colour.Green()
        target_blue = self.base_colour.Blue()

        h,s,v = colorsys.rgb_to_hsv(target_red / 255.0, target_green / 255.0,
                                    target_blue / 255.0)
        v = 1.0
        vstep = 1.0 / self.HEIGHT
        for y_pos in range(0, self.HEIGHT):
            r,g,b = [c * 255.0 for c in colorsys.hsv_to_rgb(h,s,v)]
            colour = wx.Colour(int(r), int(g), int(b))
            self.buffer.SetPen(wx.Pen(colour, 1, wx.PENSTYLE_SOLID))
            self.buffer.DrawRectangle(0, y_pos, 15, 1)
            v = v - vstep



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#----------------------------------------------------------------------
# Name:        wx.tools.wxget_docs_demo
# Purpose:     wx fetch and launch demo or docs
#
# Author:      Steve Barnes
#
# Created:     06-Aug-2017
# Copyright:   (c) 2017-2018 by Steve Barnes
# Licence:     wxWindows license
# Tags:        phoenix-port, py3-port
#
# Module to allow the correct version of the documents and/or demos to be
# launched after, if necessarily being fetched.
#----------------------------------------------------------------------

"""
wxget_docs_demo.py -- Launch the appropriate wx Docs or Demo.

Usage:
    wxget_docs_demo [docs|demo]

Will install if missing, the requested item for the current version and then
launch it.

Use: doc|demo --force to force a fresh download.
"""
from __future__ import (division, absolute_import, print_function, unicode_literals)

import sys
import os
import subprocess
import webbrowser
import tarfile
if sys.version_info >= (3,):
    from urllib.error import HTTPError
    import urllib.request as urllib2
    import urllib.parse as urlparse
    from urllib.request import pathname2url
else:
    import urllib2
    from urllib2 import HTTPError
    import urlparse
    from urllib import pathname2url

import wx
try:
    import wxget
except ImportError:
    from . import wxget

print(sys.version_info, sys.version, sys.argv)
APP = None
if wx.VERSION[0] != 4:
    raise ValueError("wx Versions other than 4 not currently supported!")

def ensure_wx_app():
    """ Ensure that there is a wx.App instance."""
    global APP
    if APP is None and not wx.GetApp():
        APP = wx.App()
        APP.SetAppName("wxPython")
    return (APP is not None)

def get_paths_dict():
    """ Get a dictionary of the required paths."""
    global APP
    ensure_wx_app()
    sp = wx.StandardPaths.Get()
    pathdict = {}
    pathdict['TempDir'] = sp.GetTempDir()
    pathdict['Cache'] = os.path.join(sp.GetUserLocalDataDir(), 'wxDocsDemoCache',
                                     wx.VERSION_STRING)
    pathdict['Docs_URL'] = wxget.get_docs_demo_url(False)
    #pathdict['wxDocs'] = os.path.join(sp.GetAppDocumentsDir(), 'wxDocs', wx.VERSION_STRING)
    pathdict['wxDocs'] = sp.GetAppDocumentsDir()
    pathdict['Docs_Name'] = "wxPython-docs-%s" % wx.VERSION_STRING
    pathdict['Demo_URL'] = wxget.get_docs_demo_url(True)
    #pathdict['wxDemo'] = os.path.join(sp.GetUserLocalDataDir(), 'wxDemo', wx.VERSION_STRING)
    pathdict['wxDemo'] = sp.GetUserLocalDataDir()
    pathdict['Demo_Name'] = "wxPython-demo-%s" % wx.VERSION_STRING
    pathdict['Ext'] = 'tar.gz'
    return pathdict

def unpack_cached(cached, dest_dir):
    """ Unpack from the cache."""
    print('Unpack', cached, 'to', dest_dir)
    tf = tarfile.open(cached, "r:*")
    tf.extractall(dest_dir)
    dest_dir = os.listdir(dest_dir)[0]
    return dest_dir

def get_item(final, url, cache, name, ext, forced=False):
    """ Get the item """
    print('Looking for', name, 'at', final)
    fullpath = os.path.join(final, name)
    if os.path.exists(fullpath) and not forced:  # Already exists
        return fullpath

    cached = os.path.join(cache, name)
    cached = os.path.extsep.join([cached, ext])
    print('Looking for cached', cached)
    if not os.path.exists(cached) or forced:  # No cached copy
        yes_no = wx.MessageBox(
            "\n".join(
                ["%s is not yet installed." % name,
                 "Go on-line to get it?",
                 "(Select No on charged or slow connections)"]),
            "Download Prompt", wx.YES_NO|wx.CENTER|wx.ICON_INFORMATION)
        if yes_no == wx.YES:
            cached = wxget.download_file(url, cache, force=forced, trusted=True)
        else:
            report_error("Download Cancelled!")

    if os.path.exists(cached):  # We now have a cached copy
        unpack_cached(cached, final)
    else:
        fullpath = None
    return fullpath

def report_error(err_text):
    """ Report a problem."""
    ensure_wx_app()
    wx.MessageBox(err_text, caption='ERROR!',
                  style=wx.OK|wx.CENTRE|wx.ICON_ERROR)

def done(result=0):
    """ Tidy up and exit."""
    global APP
    if APP:
        print("Closing Launcher App!")  # Debug
        if result:
            print(result)
        wx.Exit()
    print("Done!")
    sys.exit(result)

def docs_main(args=sys.argv):
    """ Get/Launch Docs."""
    ensure_wx_app()
    result = 0
    print("Launch Docs for wxPython V%s" % wx.VERSION_STRING)
    pd = get_paths_dict()
    location = get_item(pd['wxDocs'], pd['Docs_URL'], pd['Cache'],
                        pd['Docs_Name'], pd['Ext'], forced="--force" in args)
    if location:
        location = os.path.join(location, 'docs', 'html', 'index.html')
        location_url = urlparse.urljoin('file:', pathname2url(location))
        print("Show Docs at:", location)
        webbrowser.open(location_url)
    else:
        result = 'Unable to find and show the wxPython Documentation!'
        report_error(result)
    done(result)

def demo_main(args=sys.argv):
    """ Get/Launch Demo."""
    result = 0
    ensure_wx_app()
    print("Launch Demo for wxPython V%s" % wx.VERSION_STRING)
    pd = get_paths_dict()
    location = get_item(pd['wxDemo'], pd['Demo_URL'], pd['Cache'],
                        pd['Demo_Name'], pd['Ext'], forced="--force" in args)
    if location:
        cmds = [sys.executable, os.path.join(location, "demo", "demo.py")]
        print("Launching", cmds[1])
        pid = subprocess.Popen(cmds).pid
        #subprocess.check_call(cmds) # Use instead for debug
        print("Demo starting as PID %s - may take a few seconds!" % pid)
    else:
        result = 'Unable to find and start the wxPython Demo!'
        report_error(result)
    done(result)

def main(args=sys.argv):
    """ Command line main."""
    if len(args) > 1:
        if "demo" in args[1].lower():
            return demo_main()
        elif "docs" in args[1].lower():
            return docs_main()
        #else:
    print(__doc__)
    #else:
        #APP = wx.App()

if __name__ == "__main__":
    main()
